#include <ros/ros.h>
#include <schpin_lib/util.h>
#include <visualization_msgs/Marker.h>
#include <visualization_msgs/MarkerArray.h>

#pragma once

namespace schpin
{

class MarkerManager
{
        std::vector< std::pair< std::string, int > > robot_active_markers_{};
        ros::Publisher                               marker_array_pub_;

      public:
        MarkerManager( ros::NodeHandle handle )
          : marker_array_pub_(
                handle.advertise< visualization_msgs::MarkerArray >( "/visualization_marker_array", 10 ) )
        {
        }

        void publish( visualization_msgs::MarkerArray msg )
        {
                marker_array_pub_.publish( msg );
                forEach( msg.markers, [&]( const visualization_msgs::Marker& submsg ) {
                        robot_active_markers_.emplace_back( submsg.ns, submsg.id );
                } );
        }

        void clear()
        {

                visualization_msgs::MarkerArray arr;
                arr.markers = mapFtoV( robot_active_markers_, [&]( std::pair< std::string, int > p ) {
                        visualization_msgs::Marker msg;
                        msg.ns     = p.first;
                        msg.id     = p.second;
                        msg.action = visualization_msgs::Marker::DELETE;
                        return msg;
                } );
                marker_array_pub_.publish( arr );
        }

        ~MarkerManager()
        {
                clear();
        }
};

inline visualization_msgs::Marker createMarker( std::string ns, unsigned id, std::string frame, float alpha )
{
        visualization_msgs::Marker res;

        res.action       = visualization_msgs::Marker::ADD;
        res.frame_locked    = true;
        res.header.frame_id = frame;
        res.header.stamp    = ros::Time();
        res.ns              = ns;
        res.id              = int( id );

        res.pose.position.x    = 0;
        res.pose.position.y    = 0;
        res.pose.position.z    = 0;
        res.pose.orientation.x = 0.0;
        res.pose.orientation.y = 0.0;
        res.pose.orientation.z = 0.0;
        res.pose.orientation.w = 1.0;

        res.color.a = alpha;

        res.lifetime = ros::Duration();

        return res;
}

}  // namespace schpin
