#include <iterator>

#pragma once

namespace schpin
{
template < typename T >
class NumericIterator
{
        T val_;

      public:
        using value_type        = T;
        using difference_type   = std::ptrdiff_t;
        using pointer           = T*;
        using reference         = T&;
        using iterator_category = std::bidirectional_iterator_tag;

        NumericIterator( T val )
          : val_( val )
        {
        }

        T operator*() const
        {
            return val_;
        }

        NumericIterator operator++( int )
        {
                auto copy( *this );
                val_++;
                return copy;
        }

        NumericIterator operator--( int )
        {
                auto copy( *this );
                val_--;
                return copy;
        }
        NumericIterator& operator++()
        {
                ++val_;
                return *this;
        }

        NumericIterator& operator--()
        {
                --val_;
                return *this;
        }
};

template < typename T >
bool operator<( const NumericIterator< T >& lh, const NumericIterator< T >& rh )
{
        return *lh < *rh;
}
template < typename T >
bool operator>( const NumericIterator< T >& lh, const NumericIterator< T >& rh )
{
        return *lh > *rh;
}
template < typename T >
bool operator<=( const NumericIterator< T >& lh, const NumericIterator< T >& rh )
{
        return *lh <= *rh;
}
template < typename T >
bool operator>=( const NumericIterator< T >& lh, const NumericIterator< T >& rh )
{
        return *lh >= *rh;
}

template < typename T >
bool operator==( const NumericIterator< T >& lh, const NumericIterator< T >& rh )
{
        return *lh == *rh;
}

template < typename T >
bool operator!=( const NumericIterator< T >& lh, const NumericIterator< T >& rh )
{
        return !( lh == rh );
}

template < typename Iter >
class IteratorIterator
{
        Iter iterator_;

      public:
        using value_type        = Iter;
        using difference_type   = std::ptrdiff_t;
        using pointer           = Iter*;
        using reference         = Iter&;
        using iterator_category = std::bidirectional_iterator_tag;

        IteratorIterator( Iter iterator )
          : iterator_( iterator )
        {
        }

        const Iter& operator*() const
        {
                return iterator_;
        }
        reference operator*()
        {
                return iterator_;
        }

        pointer operator->() const
        {
                return *iterator_;
        };

        IteratorIterator& operator++()
        {
                ++iterator_;
                return *this;
        }

        IteratorIterator& operator--()
        {
                --iterator_;
                return *this;
        }
        IteratorIterator& operator++( int )
        {
                auto copy( *this );
                iterator_++;
                return copy;
        }

        IteratorIterator& operator--( int )
        {
                auto copy( *this );
                iterator_--;
                return copy;
        }
};

template < typename Iter >
bool operator==( const IteratorIterator< Iter >& lh, const IteratorIterator< Iter >& rh )
{
        return *lh == *rh;
}
template < typename Iter >
bool operator!=( const IteratorIterator< Iter >& lh, const IteratorIterator< Iter >& rh )
{
        return !( lh == rh );
}

}  // namespace schpin
