#include "schpin_lib/console.h"

namespace schpin
{

using namespace console;

const CompletionContext* COMPLETION_CANDIDATES = nullptr;

template < typename Container >
const CompletionContext* match_context( Container views, const CompletionContext* node )
{
        SCHPIN_ASSERT( node != nullptr );

        if ( views.empty() ) {
                return node;
        }

        auto first = views.front();

        auto iter = findIf( node->children, [&]( const CompletionContext* context ) {
                return context->key.compare( 0, first.size(), first );
        } );

        if ( iter == node->children.end() ) {
                return nullptr;
        }

        return match_context( tail( views ), *iter );
}

char* completion_generator( const char* text, int state )
{

        using Iter = std::vector< const CompletionContext* >::const_iterator;
        // TODO not thread safe!
        static Range< Iter >    candidates;
        static std::string_view word;

        // is '0' when this is called fist time for 'text'
        if ( state == 0 ) {
                // we need split by all whitespaces that removes empty words...
                std::vector< std::string_view > words = split( text );
                if ( words.empty() ) {
                        return nullptr;
                }
                word = words.back();
                words.pop_back();
                auto node  = match_context( words, COMPLETION_CANDIDATES );
                candidates = range( node->children );
        }

        auto iter = findIf(
            candidates, [&]( const CompletionContext* child ) { return child->key.compare( 0, word.size(), word ); } );

        if ( iter == candidates.end() ) {
                return nullptr;
        }

        candidates = range( std::next( iter ), candidates.end() );

        return const_cast< char* >( ( *iter )->key.data() );
}

char** completer( const char* text, int /*start*/, int /*end*/ )
{
        // Don't do filename completion even if our generator finds no matches.
        rl_attempted_completion_over = 1;

        // Note: returning nullptr here will make readline use the default filename
        // completer.
        return rl_completion_matches( text, completion_generator );
}

std::vector< std::string > console::prompt( const std::vector< std::string >& p,
                                            const CompletionContext*          compl_context )
{

        COMPLETION_CANDIDATES = compl_context;

        std::stringstream ss;
        ss << Color::MAGENTA << joined( p, std::string{"  "} ) << Color::RESET << " > ";

        rl_attempted_completion_function   = completer;
        static char word_break_chars       = '\0';
        rl_completer_word_break_characters = &word_break_chars;
        std::cout << std::unitbuf;

        char* buf = readline( ss.str().c_str() );

        if ( buf == nullptr ) {
                return {};
        }

        if ( strlen( buf ) == 0 ) {
                free( buf );
                return {};
        }

        add_history( buf );

        std::vector< std::string > res =
            mapFtoV( split( buf ), [&]( std::string_view view ) { return std::string{view}; } );
        free( buf );
        return res;
}

}  // namespace schpin
