#include "schpin_lib/chain.h"
#include <gtest/gtest.h>
#include <numeric>

using namespace schpin;

TEST( Chain, iterator )
{
        ChainLinkContainer< float > links( 10 );
        std::iota( links.begin(), links.end(), 0.f );
        ChainPinContainer< int > pins( 11 );
        std::iota( pins.begin(), pins.end(), int( 0 ) );

        ChainIterator< float, int > begin{links.begin(), pins.begin()};
        ChainIterator< float, int > end{links.end(), pins.end()};

        std::for_each( begin, end, [&]( const ChainView< float, int >& view ) {
                EXPECT_EQ( view.source_pin, view.link );
                EXPECT_EQ( view.target_pin, view.link + 1 );
        } );

        ASSERT_EQ( std::distance( begin, end ), links.size() );

        auto iter = std::find_if( begin, end, []( const ChainView< float, int >& ) { return false; } );
        ASSERT_EQ( iter, end );
}

TEST( Chain, chainPushBack )
{
        Chain< float, int > chain;

        chain.pushBack( int( 0 ) );

        ASSERT_EQ( chain.size(), 0 );

        chain.pushBack( int( 1 ) );

        ASSERT_EQ( chain.size(), 1 );
}

TEST( Chain, chainInsert )
{
        Chain< float, int > chain;

        std::vector< int > pins( 10 );
        std::iota( pins.begin(), pins.end(), int( 0 ) );

        auto iter = chain.insert( chain.end(), pins );

        ASSERT_EQ( iter, chain.begin() );
        ASSERT_EQ( chain.size(), 9 );

        iter = chain.insert( chain.end(), int( 10 ) );

        ASSERT_EQ( iter, std::prev( chain.end() ) );
        ASSERT_EQ( chain.size(), 10 );
}
