__author__ = 'squirrel'

from math import *
from tf import transformations
import numpy as np

def rotate_vector(v, q):
    q2 = list(v)
    q2.append(0.0)
    return transformations.quaternion_multiply(
        transformations.quaternion_multiply(q, q2),
        transformations.quaternion_conjugate(q)
    )[:3]

def calculate_angles(body, base, coxa, tibia, femur, target):
    # in step A we modify target, to negate effects of center/orientation/base vectors, so we can assume them being [0,0,0]

    # negate offset rotation of leg on the base of the robot
    t = rotate_vector(target, transformations.quaternion_conjugate(base['orientation']))

    # negate offset leg on the base of the robot
    t = np.subtract(t, base['position'])

    # negate effect of rotation of the base
    t = rotate_vector(t, transformations.quaternion_conjugate(body['orientation']))

    #negate offset of the center of robot from [0,0,0]
    t = np.subtract(t, body['position'])

    return centered_angles(coxa, tibia, femur, t)

def centered_angles(coxa, tibia, femur, target):
    # at first we get target for the case that angles are 0,0,0
    zero_target = np.sum([coxa, tibia, femur])

    # we calculate different between zero an actual angle -> angle we should actually move
    alfa = atan2(target[1], target[0]) - atan2(zero_target[1], zero_target[0])

    # we create quaternion that can negate that angle
    alfa_q = transformations.quaternion_about_axis(-alfa, [0, 0, 1])

    # we negate that angle and use plane_angles to calculate beta/gama angles :)
    beta, gama = plane_angles(coxa, tibia, femur, rotate_vector(target, alfa_q))

    return alfa, beta, gama

def plane_angles(coxa, tibia, femur, target):
    # we negate the effect of coxa offset and assert alfa = 0 from now on :)

    t = np.subtract(target, coxa)

    return centered_plane_angles(tibia, femur, t)

def centered_plane_angles(tibia, femur, target):
    target_l = sqrt(target[0]**2 + target[2]**2)

    femur_l = sqrt(femur[0]**2 + femur[2]**2)
    tibia_l = sqrt(tibia[0]**2 + tibia[2]**2)


    # as base we use the angle 'target' has against the [0,0]
    beta = atan2(target[2], target[0])

    # we add inner angle of our calculated triangle
    beta += acos(
        -(tibia_l**2 - femur_l**2 - target_l**2)
        /
        (2*femur_l*sqrt(target_l**2))
    )

    # last step is to compensate the fact that tibia 0 angle doesn't have
    # to be x+ axis. 'tibia' arg should be where arm points at 0 angle,
    # via atan2 we figure out where that is compared to x+ axis
    # and we add negation of it to our actual goal :)
    beta -= atan2(tibia[2], tibia[0])


    gama = acos(
        -(target_l**2 - femur_l**2 - tibia_l**2)
        /
        (2*femur_l*tibia_l)
    )

    # gama is now value 'a' from 0 to 180, as angle inside the triangle
    # which we use to calculate where femur should be at.
    #       femur
    #      .-----.T   We actually need angle 'b' from it.. and assert that
    #      |a   /     x axis in the graph is the 0 point, so we calculate
    # tibia|   /      b = -(2*pi-a)    x+
    #      |  /                      |
    #      | /                       |b
    #      |/                        .-----.T
    #      .                         |a   /
    #
    # no b is angle from x+ axis, which or axis should be at
    gama = -(2*pi-gama)

    # last step is to compensate the fact that femur 0 angle doesn't have
    # to be x+ axis. 'femur' arg should be where arm points at 0 angle,
    # via atan2 we figure out where that is compared to x+ axis
    # and we add negation of it to our actual goal :)
    gama -= atan2(femur[2], femur[0])

    return beta, gama
