#!/usr/bin/python2
import unittest
import schpin.scripts.algo as algo

from math import *

class TestLegAlgorithms(unittest.TestCase):

    coxa_v  = [1, 0, -1]
    tibia_v = [1, 0, 0]
    femur_v = [1, 0, 0]

    def test_centered_plane_angles(self):

        #   target_v    beta   gama
        cases = [
            ([1, 0, 0], pi/3, pi/3),
            ([2, 0, 0], 0, pi),
            ([1, 0, 1], pi/2, pi/2)
        ]

        for target, t_beta, t_gama in cases:

            beta, gama = algo.centered_plane_angles(self.tibia_v, self.femur_v, target)

            self.assertAlmostEqual(
                first=beta,
                second=t_beta,
                places=8,
                msg="vectors: {} beta: {} target beta: {}".format(
                    {"tibia":self.tibia_v, "femur":self.femur_v, "target":target},
                    beta,
                    t_beta
                )
            )

            self.assertAlmostEqual(
                first=gama,
                second=t_gama,
                places=8,
                msg="vectors: {} gama: {} target gama: {}".format(
                    {"tibia":self.tibia_v, "femur":self.femur_v, "target":target},
                    gama,
                    t_gama
                )
            )

    def test_plane_angles(self):

        #   target_v    beta   gama
        cases = [
            ([2, 0, -1], pi/3, pi/3),
            ([3, 0, -1], 0, pi),
            ([2, 0, 0], pi/2, pi/2)
        ]


        for target, t_beta, t_gama in cases:

            beta, gama = algo.plane_angles(self.coxa_v,self.tibia_v, self.femur_v, target)

            self.assertAlmostEqual(
                first=beta,
                second=t_beta,
                places=8,
                msg="vectors: {} beta: {} target beta: {}".format(
                    {"coxa":self.coxa_v, "tibia":self.tibia_v, "femur":self.femur_v, "target":target},
                    beta,
                    t_beta
                )
            )

            self.assertAlmostEqual(
                first=gama,
                second=t_gama,
                places=8,
                msg="vectors: {} gama: {} target gama: {}".format(
                    {"coxa":self.coxa_v, "tibia":self.tibia_v, "femur":self.femur_v, "target":target},
                    gama,
                    t_gama
                )
            )

    def test_centered_angles(self):

        

if __name__ == '__main__':
    unittest.main(verbosity=2)