
#include "base/point.h"

#pragma once

namespace schpin
{

template < unsigned N >
class Line
{
        std::array< Point< N >, 2 > points_;

      public:
        using ConstIterator = const Point< N >*;

        Line( const Point< N >& from, const Point< N >& to )
          : points_( {from, to} )
        {
        }

        ConstIterator begin() const
        {
                return points_.begin();
        }

        ConstIterator end() const
        {
                return points_.end();
        }

        const Point< N >& operator[]( std::size_t i ) const
        {
                return points_[i];
        }
};

template < unsigned N >
inline Distance linePointDistance( const Line< N >& line, const Point< N >& p )
{
        Point< N > direction = line[1] - line[0];
        auto       l2        = direction.length2();
        if ( *l2 == 0.f ) {
                return p.distance( line[0] );
        }
        float projection_dist = float( ( p - line[0] ).dot( direction ) );
        projection_dist /= *l2;
        projection_dist = schpin::clamp( 0.f, projection_dist, 1.f );

        Point< N > closest_p = line[0] + projection_dist * direction;

        return closest_p.distance( p );
}

template < unsigned N >
class LineCollisionQuery : ICollisionItem< const Point< N >*, typename std::vector< Point< N > >::const_iterator >
{
        using AxesContainer = std::vector< Point< N > >;

        const Line< N >&     line_;
        const AxesContainer& axes_;

      public:
        explicit LineCollisionQuery( const Line< N >& line, const std::vector< Point< N > >& axes )
          : line_( line )
          , axes_( axes )
        {
        }

        using ItemType               = Line< N >;
        using PointIterator          = const Point< N >*;
        using SeparationAxisIterator = typename AxesContainer::const_iterator;

        Range< PointIterator > getPoints() const override
        {
                return range( line_ );
        }

        Range< SeparationAxisIterator > getSeparationAxes() const override
        {
                return range( axes_ );
        }

        Range< SeparationAxisIterator > getIntersectionAxes() const override
        {
                return {axes_.end(), axes_.end()};
        }
};

}  // namespace schpin
