#include <array>
#include <cmath>

#pragma once

namespace schpin
{

constexpr float DEFAULT_EPSILON = 1.19e-07f;

template < typename T >
inline bool almostEqual( const T& lh, const T& rh, float eps = DEFAULT_EPSILON )
{
        return float( std::abs( lh - rh ) ) < eps;
}

template < class T >
const T& max( const T& a, const T& b )
{
        return ( a < b ) ? b : a;
}
template < class T >
const T& min( const T& a, const T& b )
{
        return ( a < b ) ? a : b;
}

constexpr float fac( float N )
{
        if ( N == 1 ) {
                return N;
        }

        return N * fac( N - 1 );
}

template < unsigned N >
struct detcalc
{
        static float determinant( const std::array< float, N * N >& matrix )
        {
                std::array< float, ( N - 1 ) * ( N - 1 ) > submatrix{};
                float                                      res = 0;
                for ( unsigned i = 0; i < N; ++i ) {
                        for ( unsigned x = 0; x < N - 1; ++x ) {
                                for ( unsigned y = 1; y < N; ++y ) {
                                        if ( x < i ) {
                                                submatrix.at( x + ( y - 1 ) * ( N - 1 ) ) = matrix.at( x + y * N );
                                        } else {
                                                submatrix.at( x + ( y - 1 ) * ( N - 1 ) ) = matrix.at( 1 + x + y * N );
                                        }
                                }
                        }
                        res += ( i % 2 == 0 ? 1 : -1 ) * matrix[i] * detcalc< N - 1 >::determinant( submatrix );
                }
                return res;
        }
};

template <>
struct detcalc< 2 >
{
        static float determinant( const std::array< float, 2 * 2 >& matrix )
        {
                return matrix[0] * matrix[3] - matrix[1] * matrix[2];
        }
};

template < unsigned N >
float determinant( const std::array< float, N * N >& matrix )
{
        return detcalc< N >::determinant( matrix );
}

}  // namespace schpin
