#include "schpin_lib/error.h"
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <filesystem>

#pragma once

namespace schpin
{

namespace pt = ::boost::property_tree;

struct XMLStorageErrorTag
{
};

using XMLStorageError = Error< XMLStorageErrorTag >;

template < typename T, typename UnaryFunction >
Either< T, XMLStorageError > safeLoadXml( std::istream& sstream, UnaryFunction f )
{
        pt::ptree tree;
        try {
                pt::read_xml( sstream, tree );
        }
        catch ( const pt::xml_parser::xml_parser_error& e ) {
                return {E( XMLStorageError ) << "Failed to parse the SRDF file:" << e.what()};
        }

        try {
                return {f( tree )};
        }
        catch ( const pt::ptree_error& e ) {
                return {E( XMLStorageError ) << "Failed to extract SRDF data:" << e.what()};
        }
}
}  // namespace schpin
