#include "body.h"
#include "octree/insert.h"
#include "schpin_lib/segment_view.h"

namespace schpin
{

OcTree< OccupancyContent > createBodyOccupancyTree( const std::vector< Triangle< 3 > >& surface, Length edge_length )
{
        OcTree< OccupancyContent > res( OccupancyContent( OC_MIN_OCCUPANCY ), edge_length );

        forEach( surface, [&]( const Triangle< 3 >& t ) {
                forEach( t, [&]( const Point< 3 >& p ) { res.expand( p ); } );

                FullSingleTriangleInsert visitor( t, OccupancyToken::FULL );
                res.modify< FullSingleTriangleInsert >( visitor );
        } );

        return res;
}
Either< Body, BodyBootError > loadBody( const urdf::Model& model, const std::string& link_name )
{
        urdf::LinkConstSharedPtr link = model.getLink( link_name );

        if ( !link ) {
                return {E( BodyBootError ) << "Failed to load the body link;"};
        }

        return extractModel( *link )
            .convertLeft( [&]( StructureModel&& model ) {
                    MinMax< Point< 3 > > bound_cube = minmax( makeSegmentizedRange( model.getSurface() ) );
                    SCHPIN_INFO_LOG( "Body surface bounding cube: " << bound_cube.min << ", " << bound_cube.max );
                    return Body( std::move( model ), Length( 0.05f ) );
            } )
            .convertRight( [&]( UrdfParseError error ) {
                    return E( BodyBootError ).attach( error ) << "Body not loaded due to error in urdf "
                                                                 "parsing";
            } );
}
}  // namespace schpin
