#include "octree/occupancy.h"
#include "octree/tree.h"
#include "smodel.h"

#pragma once

namespace schpin
{

OcTree< OccupancyContent > createBodyOccupancyTree( const std::vector< Triangle< 3 > >& surface, Length edge_length );

class Body
{
        StructureModel             model_;
        OcTree< OccupancyContent > collision_tree_;
        Length                     oc_edge_length_;

      public:
        explicit Body( StructureModel&& model, Length oc_edge_length )
          : model_( model )
          , collision_tree_( OccupancyContent( OC_MIN_OCCUPANCY ), oc_edge_length )
          , oc_edge_length_( oc_edge_length )
        {
                collision_tree_ = createBodyOccupancyTree( model.getSurface(), oc_edge_length );
        }

        Length getOcEdgeLength() const
        {
                return oc_edge_length_;
        }

        std::string getFrame() const
        {
                return std::string{"body"};
        }

        const StructureModel& getModel() const
        {
                return model_;
        }

        const OcTree< OccupancyContent >& getCollisionTree() const
        {
                return collision_tree_;
        }
};

using BodyBootError = Error< struct BodyBootTag >;

Either< Body, BodyBootError > loadBody( const urdf::Model& model, const std::string& link_name );

}  // namespace schpin

namespace nlohmann
{
template <>
struct adl_serializer< schpin::Body >
{
        static void to_json( nlohmann::json& j, const schpin::Body& body )
        {
                j["model"]          = body.getModel();
                j["oc_edge_length"] = body.getOcEdgeLength();
        }

        static schpin::Body from_json( const nlohmann::json& j )
        {
                return schpin::Body( j["model"].get< schpin::StructureModel >(),
                                     j["oc_edge_length"].get< schpin::Length >() );
        }
};
}  // namespace nlohmann
