#include "body/base.h"
#include "body/vertex.h"
#include "graph/interface.h"
#include <schpin_lib/identificator_container.h>
#include <utility>
#include <vector>

#pragma once

namespace schpin
{

template < typename VertexContentT >
class BodyGraph : IGraph< BodyVertex< VertexContentT >, BodyEdge >
{
      public:
        using VertexContent       = VertexContentT;
        using Vertex              = BodyVertex< VertexContent >;
        using VertexID            = BodyVertexID;
        using VertexContainer     = Container< Vertex, VertexID, std::vector >;
        using ConstVertexIterator = typename VertexContainer::const_iterator;
        using Edge                = BodyEdge;
        using EdgeID              = BodyEdgeID;
        using Price               = BodyPrice;
        using GraphDistance       = BodyPrice;

      private:
        VertexContainer vertexes_;

      public:
        BodyGraph()
          : vertexes_(){};

        Range< ConstVertexIterator > getVertexes() const
        {
                return range(vertexes_);
        }

        const Edge& getEdge( const EdgeID& eid ) const override
        {
                return getVertex( eid.vertex_id ).getEdge( eid.edge_offset );
        }

        const Vertex& getVertex( const VertexID& id ) const override
        {
                return vertexes_[id];
        }

        VertexContent& refContent( const VertexID& id ) override
        {
                return *vertexes_[id];
        }

        template < typename UnaryFunction >
        void modifyAll( UnaryFunction f )
        {
                forEach( vertexes_, [&]( Vertex& v ) { f( *v ); } );
        }

        const Vertex& emplaceVertex( VertexContent content )
        {
                VertexID id{vertexes_.size()};
                vertexes_.emplaceBack( id, std::move( content ) );
                return vertexes_[id];
        }

        std::size_t size() const
        {
                return std::size_t{vertexes_.size()};
        }

        template < typename UnaryFunction >
        void addEdge( VertexID source, VertexID target, UnaryFunction f )
        {
                BodyEdgeID eid = vertexes_[source].addEdge( f( source, target ), target );
                vertexes_[target].addReverseEdge( eid );
        }
};

}  // namespace schpin
