#include "base/point.h"
#include "base/quaternion.h"
#include <random>

namespace schpin
{
class BodyGenerator
{
        std::random_device                       rd_{};
        std::mt19937                             gen_{rd_()};
        std::normal_distribution< float >        normal_d_{0, 1};
        std::uniform_real_distribution< double > uniform_d_{double( *-PI ), double( *PI )};

      public:
        Pose getRandomBodyCoord( const Pose& from, const Pose& to )
        {

            // TODO: to debug the maze: gnerate just random points, without collision detection!
                Pose center = interp( from, to, 0.5f );

                Length l = center.distance( to ).dist;

                Point< 3 > offset{
                    normal_d_( gen_ ) * l / 1.f,
                    normal_d_( gen_ ) * l / 2.f,
                    0.f,
                };

                Quaternion direction =
                    shortestArcQuat( ( to.position - from.position ).normalized(), Point< 3 >( 1, 0, 0 ) );

                Quaternion rot{tf::createQuaternionFromRPY( 0., 0., uniform_d_( gen_ ) )};

                Point< 3 > new_coord = center.position + rotate( offset, direction );

                Distance a = new_coord.distance( to.position );
                Distance b = new_coord.distance( from.position );

                float r = *b / ( *a + *b );

                return Pose{new_coord, from.orientation.slerp( to.orientation, r ) * rot};
        }
};
}  // namespace schpin
