#include "base/pose.h"
#include <vector>

#pragma once

namespace schpin
{
class BodyViewSegment
{
      private:
        std::vector< Pose > steps_;

      public:
        using const_iterator = typename std::vector< Pose >::const_iterator;
        using iterator       = typename std::vector< Pose >::iterator;

        BodyViewSegment() = default;

        BodyViewSegment( Pose from, Pose to, Angle angle_step, Distance distance_step )
          : steps_()
        {
                float steps_float = *max( from.position.distance( to.position ) / distance_step,
                                          from.orientation.angleShortestPath( to.orientation ) / angle_step );
                auto  steps       = std::size_t( steps_float );
                steps_            = mapFtoV( range( steps ),
                                  [&]( std::size_t i ) {  //
                                          return interp( from, to, float( i ) / float( steps ) );
                                  } );
        }

        Range< const_iterator > getPoses() const
        {
                return range( steps_ );
        }
};
}  // namespace schpin
