#include "body/segment.h"
#include "gait/chain.h"
#include "schpin_lib/segment_view.h"
#include <utility>

#pragma once

namespace schpin
{

template < typename GaitGraph >
class BodyView
{

        // TODO: code duplication of LegView ....
      public:
        using PoseIterator = typename BodyViewSegment::const_iterator;
        using BodyChain    = GaitChain< typename GaitGraph::Vertex >;
        using ChainIter    = typename BodyChain::const_iterator;
        using ChainView    = typename BodyChain::ConstView;
        using InnerIter    = SegmentizedIterator< ChainIter, PoseIterator >;

      private:
        std::function< Range< PoseIterator >( const ChainView& ) > f_;
        Range< InnerIter >                                         range_;
        std::vector< unsigned >                                    steps_;
        int                                                        first_;

      public:
        BodyView( const BodyChain& chain, std::vector< unsigned > steps, int first )
          : f_( []( const ChainView& view ) -> Range< PoseIterator > {
                  return range( view.link.body_segment.getPoses() );
          } )
          , range_( makeSegmentizedRange( chain, f_ ) )
          , steps_( std::move( steps ) )
          , first_( first )
        {
        }

        // TODO(squirrel): no begin/end error checking

        std::vector< Pose > getCandidateOffsets( const Pose& body_pose ) const
        {
                std::vector< Pose > res;
                InnerIter           iter = findClosest( body_pose );

                iter = std::next( iter, first_ );
                SCHPIN_ASSERT( iter != range_.end() );

                res.push_back( *iter );

                forEach( steps_, [&]( unsigned step ) {
                        iter = std::next( iter, step );
                        res.push_back( *iter );
                } );

                return res;
        }

      private:
        InnerIter findClosest( const Pose& pose ) const
        {
                return minIterator( range_, [&]( const Pose& p ) {
                        PoseDistance d = p.distance( pose );
                        // TODO: random constant...
                        return *d.dist + ( *d.angle ) * 0.1f;
                } );
        }
};
}  // namespace schpin
