#include <schpin_lib/util.h>

#pragma once

namespace schpin
{

template < typename Query >
class SimpleCollisionTreeNode : ICollisionItem< typename Query::PointIterator, typename Query::SeparationAxisIterator >
{
      public:
        using SeparationAxisIterator = typename Query::SeparationAxisIterator;
        using PointIterator          = typename Query::PointIterator;

      private:
        static_assert( std::is_base_of_v< ICollisionItem< PointIterator, SeparationAxisIterator >, Query > );

        const Query query_;

      public:
        explicit SimpleCollisionTreeNode( const Query& query )
          : query_( query )
        {
        }
        Range< SeparationAxisIterator > getSeparationAxes() const override
        {
                return query_.getSeparationAxes();
        }

        Range< PointIterator > getPoints() const override
        {
                return query_.getPoints();
        }

        Range< SeparationAxisIterator > getIntersectionAxes() const override
        {
                return query_.getIntersectionAxes();
        }

        bool isLeaf() const
        {
                return true;
        }

        Range< const SimpleCollisionTreeNode* > getChildren() const
        {
                return {nullptr, nullptr};
        }
};

template < typename Query >
class FastCollisionTreeNode : IFastCollisionItem< typename Query::Full, Query::N >
{
        const Query query_;

      public:
        static constexpr unsigned N = Query::N;
        using Full                  = typename Query::Full;

        explicit FastCollisionTreeNode( const Query& query )
          : query_( query )
        {
        }

        Point< N > getMinSpherePosition() const override
        {
                return query_.getMinSpherePosition();
        }

        Radius getMinSphereRadius() const override
        {
                return query_.getMinSphereRadius();
        }

        bool isLeaf() const
        {
                return true;
        }

        Range< const FastCollisionTreeNode* > getChildren() const
        {
                return {nullptr, nullptr};
        }

        Full getFull() const override
        {
                return query_.getFull();
        }
};

}  // namespace schpin
