
#include "base/physics.h"
#include "util.h"

#pragma once

namespace schpin
{

enum class CollisionMode
{
        FULL,
        FAST
};

template < typename PointIterator, typename SeparationAxisIterator >
class ICollisionItem  // NOLINT(cppcoreguidelines-special-member-functions)
{

      public:
        virtual Range< PointIterator >          getPoints() const           = 0;
        virtual Range< SeparationAxisIterator > getSeparationAxes() const   = 0;
        virtual Range< SeparationAxisIterator > getIntersectionAxes() const = 0;
        virtual ~ICollisionItem()                                           = default;
};

template < unsigned N >
class Point;

template < typename FullT, unsigned Nu >
class IFastCollisionItem
{
      public:
        using Full                  = FullT;
        static constexpr unsigned N = Nu;

        virtual Point< N > getMinSpherePosition() const = 0;
        virtual Radius     getMinSphereRadius() const   = 0;
        virtual Full       getFull() const              = 0;
        virtual ~IFastCollisionItem()                   = default;
};

}  // namespace schpin
