#include "environment/octree.h"
#include "octree/load.h"
#include "octree/occupancy.h"
#include "octree/tree.h"
#include "schpin_lib/either.h"
#include "schpin_lib/error.h"
#include <filesystem>

#pragma once

namespace schpin
{

using EnvProxyNode = OcProxyNode< EnvironmentOcTreeContent >;

class Environment
{
        OcTree< EnvironmentOcTreeContent > tree_;

      public:
        explicit Environment( OcTree< EnvironmentOcTreeContent >&& tree )
          : tree_( std::move( tree ) )
        {
        }

        const OcTree< EnvironmentOcTreeContent >& getTree() const
        {
                return tree_;
        }
};

inline std::ostream& operator<<( std::ostream& os, const Environment& /*env*/ )
{
        return os;
}

using EnvError = Error< struct EnvErrorTag >;

struct EnvConfig
{
        uint16_t                             default_occ;
        Length                               min_edge_length;
        std::vector< std::filesystem::path > paths;
};

Either< Environment, EnvError > buildEnvironment( const EnvConfig& config );

}  // namespace schpin
namespace nlohmann
{

template <>
struct adl_serializer< schpin::Environment >
{
        static void to_json( nlohmann::json& j, const schpin::Environment& env )
        {
                j["tree"] = env.getTree();
        }

        static schpin::Environment from_json( const nlohmann::json& j )
        {
                const nlohmann::json& j_tree = j["tree"];
                return schpin::Environment( j_tree.get< schpin::OcTree< schpin::EnvironmentOcTreeContent > >() );
        }
};
}  // namespace nlohmann
