#include <schpin_lib/identificator.h>
#include <schpin_lib/quantity.h>

#pragma once

namespace schpin
{

using GaitSubVertexID = Identificator< struct GaitSubVertexIDTag, 32 >;

struct GaitPriceTag : QuantityTag
{
};
using GaitPrice = Quantity< NeutralRatio, GaitPriceTag, float >;

using GaitEdgeOffset = Identificator< struct GaitEdgeOffsetTag, 32 >;

struct GaitVertexID
{
        BodyVertexID    bid;
        GaitSubVertexID gid;
};

inline bool operator==( const GaitVertexID& lh, const GaitVertexID& rh )
{
        return lh.bid == rh.bid && lh.gid == rh.gid;
}

inline bool operator!=( const GaitVertexID& lh, const GaitVertexID& rh )
{
        return !( lh == rh );
}

inline bool operator<( const GaitVertexID& lh, const GaitVertexID& rh )
{
        if ( lh.bid == rh.bid ) {
                return lh.gid < rh.gid;
        }
        return lh.bid < rh.bid;
}

struct GaitEdgeID
{
        GaitVertexID   vertex_id;
        GaitEdgeOffset edge_offset;
};

inline bool operator==( const GaitEdgeID& lh, const GaitEdgeID& rh )
{
        return lh.vertex_id == rh.vertex_id && lh.edge_offset == rh.edge_offset;
}

}  // namespace schpin
