#include "body/base.h"
#include "body/segment.h"
#include "gait/vertex.h"
#include "leg/sidepath.h"
#include "schpin_lib/chain.h"
#include <utility>

#pragma once

namespace schpin
{
struct GaitPin
{
        Pose         body_pose;
        BodyVertexID bid;
};

template < typename Vertex >
struct GaitLink
{
        using VertexContainer = Container< Vertex, GaitSubVertexID, std::vector >;
        using VertexContent   = typename Vertex::Content;

        VertexContainer                    vertexes;
        LegContainer< LegSidepathSegment > sidepaths_segments;
        BodyViewSegment                    body_segment;

        GaitLink() = default;

        GaitLink( LegContainer< LegSidepathSegment >&& sidepaths_segments, BodyViewSegment body_segment )
          : sidepaths_segments( std::move( sidepaths_segments ) )
          , body_segment( std::move( body_segment ) )
        {
        }
};

template < typename Vertex >
using GaitChain = Chain< GaitLink< Vertex >, GaitPin >;
template < typename Vertex >
using GaitChainView = ChainView< GaitLink< Vertex >, GaitPin >;
template < typename Vertex >
using GaitChainConstView = ChainView< const GaitLink< Vertex >, const GaitPin >;

template < typename ChainView >
inline BodyVertexID getGaitViewID( const ChainView& view )
{
        return view.source_pin.bid;
}

}  // namespace schpin
