#include "gait/graph.h"
#include "leg/sidepath.h"
#include "schpin_lib/segment_view.h"
#include <utility>

#pragma once

namespace schpin
{

template < typename GaitGraph >
class LegSidepathView
{
      public:
        using PointIterator          = typename LegSidepathSegment::const_iterator;
        using GaitVertex             = typename GaitGraph::Vertex;
        using GaitChainConstIterator = typename GaitChain< GaitVertex >::const_iterator;

        using InnerIter = SegmentizedIterator< GaitChainConstIterator, PointIterator >;

      private:
        Range< InnerIter >      range_;
        std::vector< unsigned > steps_;
        int                     first_;

      public:
        LegSidepathView( LegID lid, const GaitChain< GaitVertex >& chain, std::vector< unsigned > steps, int first )
          : range_(
                makeSegmentizedRange( chain,
                                      [lid]( const GaitChainConstView< GaitVertex >& view ) -> Range< PointIterator > {
                                              return view.link.sidepaths_segments[lid].getPoints();
                                      } ) )
          , steps_( std::move( steps ) )
          , first_( first )
        {
        }

        Range< InnerIter > getPoints()
        {
                return range_;
        }

        std::vector< Point< 3 > > getCandidatePositions( const Point< 3 >& pos ) const
        {
                std::vector< Point< 3 > > res;
                InnerIter                 iter = findClosest( pos );  // maybe hint the find closest?

                iter = std::next( iter, first_ );
                SCHPIN_ASSERT( iter != range_.end() );
                // TODO(squirrel): those asserts are bad, in case rewind to first_ fails, it is necessary to know how
                // many times are missing and ignore those that are missing
                res.push_back( *iter );

                forEach( steps_, [&]( unsigned step ) {
                        iter = std::next( iter, step );
                        res.push_back( *iter );
                } );

                return res;
        }

      private:
        InnerIter findClosest( const Point< 3 >& pos ) const
        {
                return minIterator( range_, [&]( const Point< 3 > p ) { return p.distance( pos ); } );
        }
};
}  // namespace schpin
