#include <type_traits>

#pragma once

namespace schpin
{

template < typename GraphType >
class IGraphPathVisitor  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using Edge          = typename GraphType::Edge;
        using Vertex        = typename GraphType::Vertex;
        using VertexContent = typename GraphType::VertexContent;

        virtual void visitEdge( const Edge& edge )                               = 0;
        virtual void visitVertex( const Vertex& vertex, VertexContent& content ) = 0;
};

template < typename GraphType >
class IGraphProcessVisitor  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using Vertex                              = typename GraphType::Vertex;
        using VertexID                            = typename GraphType::Vertex::ID;
        virtual bool vertexReady( const Vertex& ) = 0;
        virtual void initialize( VertexID vid )   = 0;
};

template < typename GraphType >
class IGraphAffectionVisitor  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using EdgeID                                = typename GraphType::Edge::ID;
        using VertexID                              = typename GraphType::Vertex::ID;
        virtual void affectedEdge( EdgeID eid )     = 0;
        virtual void affectedVertex( VertexID vid ) = 0;
        virtual ~IGraphAffectionVisitor()           = default;
};

template < typename Content, typename Price, typename VertexID >
class IGraphEdge  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        virtual const Content& operator*() const   = 0;
        virtual const Content* operator->() const  = 0;
        virtual Content&       operator*()         = 0;
        virtual Content*       operator->()        = 0;
        virtual Price          getPrice() const    = 0;
        virtual VertexID       getSourceID() const = 0;
        virtual VertexID       getTargetID() const = 0;
        virtual ~IGraphEdge()                      = default;
};

template < typename VertexID, typename VertexContent, typename Edge >
class IGraphVertex  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using EdgeContent = typename Edge::Content;
        using EdgeOffset  = typename Edge::Offset;
        using EdgeID      = typename Edge::ID;
        //  using EdgeIterator        = decltype( std::declval< Vertex >().getEdges().begin() );
        //  using ReverseEdgeIterator = decltype( std::declval< Vertex >().getReverseEdges().begin() );

        //  virtual Range< EdgeIterator >        getEdges() const                        = 0;
        // virtual Range< ReverseEdgeIterator > getReverseEdges() const                 = 0;
        virtual const Edge&          getEdge( EdgeOffset ) const           = 0;
        virtual void                 addReverseEdge( EdgeID )              = 0;
        virtual const VertexContent& operator*() const                     = 0;
        virtual VertexContent&       operator*()                           = 0;
        virtual const VertexContent* operator->() const                    = 0;
        virtual VertexContent*       operator->()                          = 0;
        virtual VertexID             getID() const                         = 0;
        virtual EdgeID               addEdge( EdgeContent&&, VertexID id ) = 0;
        virtual EdgeContent&         refEdgeContent( EdgeOffset )          = 0;
        virtual ~IGraphVertex()                                            = default;
};

template < typename Vertex, typename Edge >
class IGraph  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using EdgeID        = typename Edge::ID;
        using EdgeContent   = typename Edge::Content;
        using VertexID      = typename Vertex::ID;
        using VertexContent = typename Vertex::Content;
        using Price         = typename Edge::Price;

        static_assert( std::is_base_of< IGraphVertex< VertexID, VertexContent, Edge >, Vertex >::value );
        static_assert( std::is_base_of< IGraphEdge< EdgeContent, Price, VertexID >, Edge >::value );

        virtual VertexContent& refContent( const VertexID& id )    = 0;
        virtual const Edge&    getEdge( const EdgeID& ) const      = 0;
        virtual const Vertex&  getVertex( const VertexID& ) const  = 0;
        //  virtual void addEdge( const VertexID source, const VertexID target, std::function< EdgeContent( VertexID,
        //  VertexID
        //  )>>);
        // virtual void modifyEdge( const EdgeID &, std::function<void(EdgeContent&)> ) = 0;
        // virtual const Vertex & emplaceVertex( args ... );
        virtual ~IGraph() = default;
};

}  // namespace schpin
