#include "base/point.h"
#include "schpin_lib/rviz.h"

#pragma once

namespace schpin
{

inline visualization_msgs::Marker createVertexMarker( unsigned id, std::string frame )
{
        visualization_msgs::Marker res = createMarker( "vertex" + frame, id, frame, 1.f );

        res.type = visualization_msgs::Marker::SPHERE_LIST;

        res.scale.x = 0.002;
        res.scale.y = 0.002;
        res.scale.z = 0.002;

        res.color.r = 1.f;
        res.color.g = 0.f;
        res.color.b = 0.f;

        return res;
}
inline visualization_msgs::Marker createEdgeMarker( unsigned id, std::string frame )
{
        visualization_msgs::Marker res = createMarker( "edge" + frame, id, frame, 0.5f );

        res.type = visualization_msgs::Marker::LINE_LIST;

        res.scale.x = 0.001;

        res.color.r = 1.f;
        res.color.g = 0.f;
        res.color.b = 0.f;

        return res;
}

template < typename Graph, typename UnaryFunction >
visualization_msgs::MarkerArray createGraphVisualization( const Graph&  g,
                                                          std::string   frame,
                                                          UnaryFunction vertex_to_point )
{
        visualization_msgs::Marker vertex_msg = createVertexMarker( 0, frame );
        visualization_msgs::Marker edge_msg   = createEdgeMarker( 0, frame );

        vertex_msg.points = mapFtoV( g.getVertexes(), [&]( const auto& vertex ) {
                Point< 3 > p = vertex_to_point( vertex );
                forEach( vertex.getEdges(), [&]( const auto& edge ) {
                        edge_msg.points.push_back( pointToMsg( p ) );
                        const auto& target_vertex = g.getVertex( edge.getTargetID() );
                        edge_msg.points.push_back( pointToMsg( vertex_to_point( target_vertex ) ) );
                } );
                return pointToMsg( p );
        } );

        visualization_msgs::MarkerArray res;
        res.markers.push_back( vertex_msg );
        res.markers.push_back( edge_msg );
        return res;
}
}  // namespace schpin
