#include <nlohmann/json.hpp>
#include <schpin_lib/identificator.h>
#include <schpin_lib/quantity.h>

#pragma once

namespace schpin
{
using LegPrice = Time;

struct LegVertexIDTag
{
};
using LegVertexID = Identificator< LegVertexIDTag, 32 >;

using LegEdgeOffset = Identificator< struct LegEdgeOffsetTag, 32 >;

struct LegEdgeID
{
        LegVertexID   vid;
        LegEdgeOffset edge_offset;

        LegEdgeID( LegVertexID id, LegEdgeOffset offset )
          : vid( id )
          , edge_offset( offset )
        {
        }
};

inline bool operator==( const LegEdgeID& lh, const LegEdgeID& rh )
{
        return lh.vid == rh.vid && lh.edge_offset == rh.edge_offset;
}

}  // namespace schpin

namespace nlohmann
{

template <>
struct adl_serializer< schpin::LegEdgeID >
{
        static void to_json( nlohmann::json& j, const schpin::LegEdgeID& eid )
        {
                j = nlohmann::json{{"vid", eid.vid}, {"offset", eid.edge_offset}};
        }

        static schpin::LegEdgeID from_json( const nlohmann::json& j )
        {
                return schpin::LegEdgeID( j["vid"].get< schpin::LegVertexID >(),
                                          j["offset"].get< schpin::LegEdgeOffset >() );
        }
};
}  // namespace nlohmann
