#include "graph/interface.h"
#include "leg/graph/base.h"
#include <nlohmann/json.hpp>

#pragma once

namespace schpin
{
class LegEdge : IGraphEdge< LegPrice, LegPrice, LegVertexID >
{

        LegEdgeID   id_;
        LegPrice    price_;
        LegVertexID to_;

      public:
        using Price   = LegPrice;
        using ID      = LegEdgeID;
        using Content = LegPrice;
        using Offset  = LegEdgeOffset;

        LegEdge( LegEdgeID id, LegPrice price, LegVertexID to )
          : id_( id )
          , price_( price )
          , to_( to )
        {
        }

        LegEdgeID getID() const
        {
                return id_;
        }
        LegPrice& operator*() override
        {
                return price_;
        }
        LegPrice* operator->() override
        {
                return &price_;
        }

        const LegPrice& operator*() const override
        {
                return price_;
        }
        const LegPrice* operator->() const override
        {
                return &price_;
        }

        LegPrice getPrice() const override
        {
                return price_;
        }

        LegVertexID getSourceID() const override
        {
                return id_.vid;
        }

        LegVertexID getTargetID() const override
        {
                return to_;
        }
};
}  // namespace schpin
namespace nlohmann
{
template <>
struct adl_serializer< schpin::LegEdge >
{
        static void to_json( nlohmann::json& json, const schpin::LegEdge& edge )
        {
                json = nlohmann::json{{"id", edge.getID()}, {"price", edge.getPrice()}, {"target", edge.getTargetID()}};
        }

        static schpin::LegEdge from_json( const nlohmann::json& j )
        {
                return schpin::LegEdge( j["id"].get< schpin::LegEdgeID >(),
                                        j["price"].get< schpin::LegPrice >(),
                                        j["target"].get< schpin::LegVertexID >() );
        }
};
}  // namespace nlohmann
