#include "leg/base.h"
#include "leg/graph.h"
#include "leg/model.h"

#pragma once

namespace schpin
{
template < unsigned i, unsigned N, typename OutputFunction >
void generateLegVertexes( DiscreteAngles< N > angles, const std::array< uint8_t, N >& steps, OutputFunction out_f )
{
        if constexpr ( i == N ) {
                out_f( angles );
        } else {
                forEach( range( steps[i] ), [&]( uint8_t step ) {
                        angles[i] = step;
                        generateLegVertexes< i + 1, N >( angles, steps, out_f );
                } );
        }
}

template < typename Graph, unsigned N >
inline Graph generateLegGraph( const std::array< uint8_t, N > steps )
{

        using Vertex        = typename Graph::Vertex;
        using VertexContent = typename Vertex::Content;
        Graph result;

        generateLegVertexes< 0, N >( DiscreteAngles< N >{}, steps, [&]( DiscreteAngles< N > angles ) {
                result.emplaceVertex( VertexContent{angles} );
        } );

        forEach( result.getVertexes(), [&]( const Vertex& vertex ) {
                forEach( result.getVertexes(), [&]( const Vertex& other ) {
                        uint8_t max_dist = max( vertex->angles - other->angles, []( uint8_t diff ) { return diff; } );

                        if ( max_dist == 1 ) {
                                result.addEdge( vertex.getID(), LegPrice( 0 ), other.getID() );
                        }
                } );
        } );

        return result;
}

template < typename Graph, unsigned N >
inline Graph generateLegGraph( const LegModel< N >& model )
{
        auto steps = mapF( model.getChain(), []( const ModelChain& chain ) { return chain.steps; } );

        return generateLegGraph< Graph, N >( steps );
}

}  // namespace schpin
