#include "base/physics.h"
#include "graph/dstar/content.h"
#include "graph/generics.h"
#include "graph/interface.h"
#include "leg/base.h"
#include "leg/graph/edge.h"
#include <schpin_lib/util.h>

#include <utility>

#pragma once

namespace schpin
{

template < typename SearchContent, unsigned N >
struct LegVertexContent
{
        SearchContent       search_content;
        DiscreteAngles< N > angles;

        LegVertexContent( DiscreteAngles< N > angles )
          : angles( angles )
        {
        }
};
}  // namespace schpin
namespace nlohmann
{
template < typename SearchContent, unsigned N >
struct adl_serializer< schpin::LegVertexContent< SearchContent, N > >
{
        static void to_json( nlohmann::json& j, const schpin::LegVertexContent< SearchContent, N >& cont )
        {
                j["angles"] = cont.angles;
        }

        static schpin::LegVertexContent< SearchContent, N > from_json( const nlohmann::json& j )
        {
                return {j["angles"].get< schpin::DiscreteAngles< N > >()};
        }
};
}  // namespace nlohmann

namespace schpin
{

template < typename VertexContent >
using LegVertex = GenericVertex< LegVertexID, VertexContent, LegEdge >;

}  // namespace schpin
