#include "leg.h"

#pragma once

namespace schpin
{

// TODO(squirrel): way a fay for this to be temporal - maybe forbid assigment operator?
template < unsigned N >
class LegView
{

        const Leg< N >& leg_;
        const Pose      body_offset_;

      public:
        LegView( const Leg< N >& leg, const Pose& body_offset )
          : leg_( leg )
          , body_offset_( body_offset )
        {
        }
        const Pose& getBodyOffset() const
        {
                return body_offset_;
        }
        LegVertexID findClosestVertex( const Point< 3 >& position, LegVertexID lid ) const
        {
                return graphFindClosestVertex(
                    position, lid, leg_.getDstar().getGraph(), [&]( const LegVertexID& id ) -> Point< 3 > {
                            return generateTipPosition( id );
                    } );
        }

        Point< 3 > generateTipPosition( LegVertexID lid ) const
        {
                const auto& vertex = leg_.getDstar().getGraph().getVertex( lid );
                return tipPosition( vertex->angles );
        }

        template < typename Angles >
        Point< 3 > tipPosition( const Angles& angles ) const
        {
                return transform( leg_.getTipPosition( angles ), body_offset_ );
        }

        template < typename Angles >
        std::vector< FastStructureModelCollisionQuery > getCollisionQueries( const Angles& angles ) const
        {
                // TODO: test this somehow
                // TODO: need indexed container :/
                auto poses         = leg_.getModel().chainPoses( angles );
                Pose global_offset = transform( leg_.getOriginOffset(), body_offset_ );
                return mapFtoV( range( angles.size() ), [&]( std::size_t i ) {
                        return FastStructureModelCollisionQuery{leg_.getModel().getChain()[i].model,
                                                                transform( poses[i], global_offset )};
                } );
        }

        const Leg< N >& operator*() const
        {
                return leg_;
        }

        const Leg< N >* operator->() const
        {
                return &leg_;
        }
};

}  // namespace schpin
