#include "base/math.h"
#include "leg/graph.h"
#include "leg/model.h"
#include "octree/insert.h"
#include "octree/occupancy.h"
#include "octree/tree.h"

#pragma once

namespace schpin
{

template < unsigned N, typename AnglesToPointFunction >
inline OcTree< OccupancyContent > createWorkarea( AnglesToPointFunction angle_converison_f, Length edge_length )
{
        using Visitor = SingleCollisionInsert< FullCollisionChecker< PointCollisionQuery< 3 >, OccupancyContent > >;

        constexpr std::size_t max_steps = 64;

        std::array< uint8_t, N > steps = mapFtoA< N >( range( N ), [&]( std::size_t ) -> uint8_t { return max_steps; } );

        OcTree< OccupancyContent > res( OccupancyContent( OC_MIN_OCCUPANCY ), edge_length );

        std::size_t count = 0;
        generateLegVertexes< 0, N >( DiscreteAngles< N >{}, steps, [&]( DiscreteAngles< N > angles ) {
                SCHPIN_INFO_LOG_DELAYED_THROTTLE( 10, "Workarea vertex count: " << count );
                Point< 3 > p = angle_converison_f( angles, max_steps );

                res.expand( p );
                Visitor vis( p, OccupancyToken::FULL );

                res.modify< Visitor >( vis );
                count++;
        } );

        return res;
}

}  // namespace schpin
