#include "base/physics.h"
#include "base/point.h"
#include <cassert>

#pragma once

namespace schpin
{

enum class OcNodeControl
{
        FOLLOW_CHILDREN,
        END
};

enum class OcLockedLeafControl
{
        EXPAND_CHILDREN,
        UNLOCK,
        END
};

class OcChildPos
{
      public:
        using IndexType = uint8_t;

        OcChildPos( Direction x, Direction y, Direction z )
          : x_( x )
          , y_( y )
          , z_( z )
        {
        }

        explicit OcChildPos( IndexType index )
          : x_( index % 2 == 0 ? Direction::NEG : Direction::POS )
          , y_( index / 2 % 2 == 0 ? Direction::NEG : Direction::POS )
          , z_( index / 4 % 2 == 0 ? Direction::NEG : Direction::POS )
        {
        }

        // converts [x,y,z] coordinate ( 0 < x,y,z < 1) into a index of child in octree
        IndexType getIndex()
        {

                return uint8_t( ( x_ == Direction::POS ? 1 : 0 ) + ( y_ == Direction::POS ? 2 : 0 ) +
                                ( z_ == Direction::POS ? 4 : 0 ) );
        }

        // based on the child index and actual edge, returns the offest of the child from it's parrent
        Point< 3 > getOffset( Length edge_length_ )
        {
                return Point< 3 >( x_ == Direction::NEG ? -edge_length_ / 4.f : edge_length_ / 4.f,
                                   y_ == Direction::NEG ? -edge_length_ / 4.f : edge_length_ / 4.f,
                                   z_ == Direction::NEG ? -edge_length_ / 4.f : edge_length_ / 4.f );
        }

      private:
        Direction x_;
        Direction y_;
        Direction z_;
};

inline OcChildPos getOcChildPosBasedOnCenter( const Point< 3 >& parent, const Point< 3 >& child )
{
        return {parent[0] > child[0] ? Direction::NEG : Direction::POS,  //
                parent[1] > child[1] ? Direction::NEG : Direction::POS,  //
                parent[2] > child[2] ? Direction::NEG : Direction::POS};
}

}  // namespace schpin
