#include "octree/proxy_node.h"

#pragma once

namespace schpin
{

template < typename Content, typename TokenT >
class IOcContent  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using Token = TokenT;

        virtual std::size_t getMemoryUsage( const OcNode< Content >& node ) const = 0;
        virtual bool        prunnable( const OcNode< Content >& node ) const      = 0;
        virtual void        childBasedUpdate( const OcNode< Content >& node )     = 0;
        virtual void        setPrunned( const OcNode< Content >& node )           = 0;
        virtual void        update( const Token& token )                          = 0;
        virtual ~IOcContent()                                                     = default;
};

// this is ugly part of interface... :)
template < typename Content, typename Msg >
OcNode< Content > msgToOctreeNode( const Msg& msg );

template < typename Content, typename MsgT >
class IOcSerializableContent  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using Msg = MsgT;

        virtual Msg toMsg( const OcNode< Content >& node ) const = 0;
        virtual ~IOcSerializableContent()                        = default;
};

template < typename NodeContentT >
class IOcCollisionCheck  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using NodeContent = NodeContentT;

        virtual bool innerCollission( const OcProxyNode< NodeContent >&,
                                      const OcNodeFullCollisionQuery< NodeContent >& ) const = 0;
        virtual void bottomCollission( const OcProxyNode< NodeContent >&,
                                       const OcNodeFullCollisionQuery< NodeContent >&,
                                       NodeContent& content ) const                          = 0;
        virtual ~IOcCollisionCheck()                                                         = default;
};

template < typename InfoType, typename NodeContent >
class IOcVisitor  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        virtual void                                       accessBottomLeaf( const OcProxyNode< NodeContent >&,
                                                                             const InfoType&,
                                                                             NodeContent& content ) const = 0;
        virtual std::pair< OcNodeControl, InfoType >       accessNode( const OcProxyNode< NodeContent >&,
                                                                       const InfoType& ) const            = 0;
        virtual std::pair< OcLockedLeafControl, InfoType > accessLockedLeaf( const OcProxyNode< NodeContent >&,
                                                                             const InfoType& ) const      = 0;
        virtual ~IOcVisitor()                                                                             = default;
};

template < typename NodeContent >
class IOcSimpleVisitor  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        virtual void          accessBottomLeaf( const OcProxyNode< NodeContent >&, NodeContent& content ) const = 0;
        virtual OcNodeControl accessNode( const OcProxyNode< NodeContent >& ) const                             = 0;
        virtual OcLockedLeafControl accessLockedLeaf( const OcProxyNode< NodeContent >& ) const                 = 0;
        virtual ~IOcSimpleVisitor() = default;
};

template < typename NodeContent >
class OcPassiveVisitor : public IOcSimpleVisitor< NodeContent >
{
      public:
        void accessBottomLeaf( const OcProxyNode< NodeContent >& /*unused*/, NodeContent& /*unused*/ ) const override
        {
        }
        OcNodeControl accessNode( const OcProxyNode< NodeContent >& /*unused*/ ) const override
        {
                return OcNodeControl::FOLLOW_CHILDREN;
        }
        OcLockedLeafControl accessLockedLeaf( const OcProxyNode< NodeContent >& /*unused*/ ) const override
        {
                return OcLockedLeafControl::END;
        }
};

struct OcEmptyInfoType
{
};

template < typename SimpleVisitor, typename NodeContent >
class OcSimpleVisitorOverlay : public IOcVisitor< OcEmptyInfoType, NodeContent >
{
        const SimpleVisitor& vis_;

      public:
        explicit OcSimpleVisitorOverlay( const SimpleVisitor& vis )
          : vis_( vis )
        {
        }
        void accessBottomLeaf( const OcProxyNode< NodeContent >& node,
                               const OcEmptyInfoType& /*unused*/,
                               NodeContent& content ) const override
        {
                vis_.accessBottomLeaf( node, content );
        }
        std::pair< OcNodeControl, OcEmptyInfoType > accessNode( const OcProxyNode< NodeContent >& node,
                                                                const OcEmptyInfoType& /*unused*/ ) const override
        {
                return {vis_.accessNode( node ), OcEmptyInfoType{}};
        }
        std::pair< OcLockedLeafControl, OcEmptyInfoType >
        accessLockedLeaf( const OcProxyNode< NodeContent >& node, const OcEmptyInfoType& /*unused*/ ) const override
        {
                return {vis_.accessLockedLeaf( node ), OcEmptyInfoType{}};
        }
};

}  // namespace schpin
