#include "base.h"
#include "base/physics.h"
#include "base/point.h"
#include <utility>

#pragma once

namespace schpin
{

class OcNodeContext
{
      public:
        OcNodeContext( unsigned depth, unsigned max_depth, Length edge_length, Point< 3 > position )
          : depth_( depth )
          , max_depth_( max_depth )
          , edge_length_( edge_length )
          , position_( std::move( position ) )
        {
        }

        unsigned getDepth() const
        {
                return depth_;
        }

        unsigned getMaxDepth() const
        {
                return max_depth_;
        }
        Length getEdgeLength() const
        {
                return edge_length_;
        }
        Point< 3 > getPosition() const
        {
                return position_;
        }

        Length getMinEdgeLength() const
        {
                return edge_length_ / float( pow( 2, max_depth_ ) );
        }

      private:
        unsigned   depth_;
        unsigned   max_depth_;
        Length     edge_length_;
        Point< 3 > position_;
};

inline std::ostream& operator<<( std::ostream& os, const OcNodeContext& cont )
{
        os << "Depth: " << cont.getDepth() << " MaxDepth: " << cont.getMaxDepth()
           << " EdgeLength: " << cont.getEdgeLength() << " Position: " << cont.getPosition();
        return os;
}

inline bool operator==( const OcNodeContext& lh, const OcNodeContext& rh )
{
        return lh.getDepth() == rh.getDepth() && lh.getMaxDepth() == rh.getMaxDepth() &&
               lh.getEdgeLength() == rh.getEdgeLength() && lh.getPosition() == rh.getPosition();
}

inline OcNodeContext upscaleContext( const OcNodeContext& context, OcChildPos index )
{
        return OcNodeContext{context.getDepth(),
                             context.getMaxDepth() + 1,
                             context.getEdgeLength() * 2.f,
                             context.getPosition() - index.getOffset( context.getEdgeLength() * 2.f )};
}

inline OcNodeContext downscaleContext( const OcNodeContext& context, uint8_t index )
{
        return OcNodeContext{context.getDepth() + 1,
                             context.getMaxDepth(),
                             context.getEdgeLength() / 2.f,
                             OcChildPos( index ).getOffset( context.getEdgeLength() ) + context.getPosition()};
}
}  // namespace schpin
