
#include "octree/proxy_node.h"
#include "octree/tree.h"
#include "schpin_lib/rviz.h"
#include <queue>
#include <ros/ros.h>

#pragma once

namespace schpin
{

inline visualization_msgs::Marker
createOcMarker( const std::string& name, unsigned id, Length scale, const std::string& frame, float alpha )
{
        visualization_msgs::Marker res = createMarker( name, id, frame, alpha );

        res.frame_locked = true;

        res.type = visualization_msgs::Marker::CUBE_LIST;

        res.action = visualization_msgs::Marker::ADD;

        res.scale.x = double( *scale );
        res.scale.y = double( *scale );
        res.scale.z = double( *scale );

        res.color.r = 0.5f;
        res.color.g = 0.5f;
        res.color.b = 0.5f;

        return res;
}

template < typename NodeContent, typename UnaryFunction >
visualization_msgs::MarkerArray createOcVisualization( const std::string&           name,
                                                       const OcTree< NodeContent >& tree,
                                                       const std::string&           frame,
                                                       float                        alpha,
                                                       UnaryFunction                filter_f )
{
        const OcProxyNode< NodeContent >         root = tree.getProxyRoot();
        std::queue< OcProxyNode< NodeContent > > que;
        visualization_msgs::MarkerArray          res;

        que.push( root );

        while ( !que.empty() ) {
                OcProxyNode< NodeContent > node = que.front();
                que.pop();

                while ( res.markers.size() <= node.getDepth() ) {
                        res.markers.push_back(
                            createOcMarker( name, node.getDepth(), node.getEdgeLength(), frame, alpha ) );
                }
                if ( !node.hasChildren() && filter_f( *node ) ) {
                        geometry_msgs::Point coord = pointToMsg( node.getPosition() );
                        res.markers[node.getDepth()].points.push_back( coord );
                }
                if ( node.hasChildren() ) {
                        forEach( node.generateChildren(),
                                 [&]( OcProxyNode< NodeContent > child ) { que.push( child ); } );
                }
        }

        return res;
}

}  // namespace schpin
