
#include "octree/base.h"
#include "octree/node.h"
#include "octree/tree.h"
#include <algorithm>

#pragma once

namespace schpin
{

// TODO:: missing templateziation

void writeOctreeNodeToFile( std::ostream& of, const OcNode& node )
{
        float occ;

        if ( node.getOccupancy() == OC_MAX_OCCUPANCY ) {
                occ = float( OC_MAX_OCCUPANCY.get() );
        } else {
                occ = float( OC_MIN_OCCUPANCY.get() );
        }
        occ = log( occ / ( 1 - occ ) );

        of.write( (char*) &occ, sizeof( float ) );

        uint8_t child_char;
        if ( node.isLeaf() ) {
                child_char = 0;
        } else {
                child_char = 255;
        }
        of.write( (char*) &child_char, sizeof( char ) );

        std::for_each(
            node.childBegin(), node.childEnd(), [&]( const OcNode& child ) { writeOctreeNodeToFile( of, child ); } );
}

void writeOctreeToFile( std::string filename, const OcTree& tree )
{

        const OcProxyNode& proxy_root = tree.getRoot< OcProxyNode >();

        Length smallest_edge_length = proxy_root.getEdgeLength() / std::pow( 2, proxy_root.getMaxDepth() );

        std::ofstream oc_file( filename.c_str(), std::ios_base::out | std::ios_base::binary );
        oc_file << "# Octomap OcTree file" << std::endl
                << "#" << std::endl
                << "id OcTree" << std::endl
                << "size " << getNodeCount( tree.getRoot() ) << std::endl
                << "res " << smallest_edge_length << std::endl
                << "data" << std::endl;

        const OcNode& root = tree.getRoot();
        writeOctreeNodeToFile( oc_file, root );

        oc_file.close();
}
}  // namespace schpin
