#include "gait/leg_sidepath.h"
#include "graph/dstar.h"
#include "leg/view.h"
#include "planner/body.h"
#include "planner/gait.h"
#include "robot.h"

#pragma once

namespace schpin
{

class Planner
{
      public:
        using GaitDStar  = AnytimeDStar< GaitDStarConfig >;
        using GaitGraph  = typename GaitDStarConfig::Graph;
        using GaitVertex = typename GaitGraph::Vertex;

      private:
        BodyPlanner< CollisionMode::FAST > body_planner_;
        GaitDStar                          gait_dstar_;
        RobotState                         robot_state_;

      public:
        Planner( PlannerContext plan_context )
          : body_planner_( std::move( plan_context ) )
        {
                robot_state_.legs = LegContainer{mapFtoV( body_planner_.getRobot().getLegs(), [&]( const Leg< 3 >& ) {
                        return LegState< 3 >{std::array< Angle, 3 >{}};
                } )};
        }

        LegSidepathView< GaitGraph > getLegSidepathView( LegID lid ) const
        {

                return {lid, gait_dstar_.getGraph().getChain(), std::vector< unsigned >{}, 0};
        }

        template < typename T >
        void bodyEvent( T t )
        {
                body_planner_.event( std::move( t ),
                                     BodyPlannerLambdaVisitor{[this]( Chain< BodyEdgeID, BodyVertexID > path ) {
                                             this->onPath( std::move( path ) );
                                     }} );
        }

        void onPath( Chain< BodyEdgeID, BodyVertexID > path )
        {
                ResetChain reset{RobotStateView( getRobot(), robot_state_ ),
                                 getEnvironment(),
                                 mapFtoV( path.getPins(), [&]( BodyVertexID bid ) -> std::pair< BodyVertexID, Pose > {
                                         return std::make_pair( bid, body_planner_.getBodyPose( bid ) );
                                 } )};

                gait_dstar_.event( std::move( reset ) );
        }

        void setGoal( const Pose& p )
        {
                body_planner_.setGoal( p );
        }

        const RobotState getRobotState() const
        {
                return robot_state_;
        }

        void setRobotState( const RobotState& rstate )
        {
                robot_state_ = rstate;
                body_planner_.setCurrent( robot_state_.body.pose );
        }

        const Environment& getEnvironment() const
        {
                return body_planner_.getEnvironment();
        }

        const Robot& getRobot() const
        {
                return body_planner_.getRobot();
        }

        const auto& getBodyPlanner() const
        {
                return body_planner_;
        }
};
}  // namespace schpin
