#include "body/state_view.h"
#include "leg/state_view.h"

#pragma once

namespace schpin
{

struct RobotState
{
        LegContainer< LegState< 3 > > legs;
        BodyState                     body;
};

class RobotStateView
{
        LegContainer< LegStateView< 3 > > legs_;
        BodyStateView                     body_;

      public:
        RobotStateView( const Robot& robot, const RobotState& state )
          : legs_()
          , body_( robot.getBody(), state.body )
        {
                legs_ = LegContainer{mapFtoV( state.legs.getIDs(), [&]( LegID id ) {
                        return LegStateView< 3 >{robot.getLegs()[id], state.legs[id]};
                } )};
        }

        const LegContainer< LegStateView< 3 > > getLegs() const
        {
                return legs_;
        }

        const BodyStateView& getBody() const
        {
                return body_;
        }

        Point< 3 > getTipPosition( LegID lid ) const
        {
                return transform( transform( legs_[lid].getTipPosition(), legs_[lid]->getOriginOffset() ),
                                  body_.getPose() );
        }
};

}  // namespace schpin
