#include "schpin_lib/util.h"
#include "schpin_lib/either.h"
#include <algorithm>
#include <chrono>
#include <iterator>
#include <utility>

#pragma once

namespace schpin
{



template < typename LH, typename RH >
struct PartitionedEithers
{
        std::vector< LH > left;
        std::vector< RH > right;
};

template < typename LH, typename RH >
PartitionedEithers< LH, RH > partitionEithers( std::vector< Either< LH, RH > >&& data )
{
        PartitionedEithers< LH, RH > res;
        for ( Either< LH, RH >& item : data ) {
                std::move( item ).match( [&]( LH&& item ) { res.left.emplace_back( std::move( item ) ); },
                                         [&]( RH&& item ) { res.right.emplace_back( std::move( item ) ); } );
        }
        return res;
}

template < typename Container, typename UnaryFunction >
void forNthIterator( unsigned N, const Container& cont, UnaryFunction f )
{
        auto begin = cont.begin();
        for ( ; begin != cont.end(); begin += std::min< int64_t >( N, std::distance( begin, cont.end() ) ) ) {
                f( begin );
        }
}

template < typename Function >
int64_t measureTime( Function f )
{
        std::chrono::high_resolution_clock::time_point t_point = std::chrono::high_resolution_clock::now();
        f();
        return std::chrono::duration_cast< std::chrono::nanoseconds >( std::chrono::high_resolution_clock::now() -
                                                                       t_point )
            .count();
}

}  // namespace schpin
