#include "leg.h"

#pragma once

using namespace schpin;

template < unsigned N >
using LegTestGraph = LegGraph< LegVertexContent< int, N > >;

template < unsigned N >
using LegTestVertex = LegVertex< LegVertexContent< int, N > >;

template < unsigned N >
LegTestGraph< N > genLegTestGraph( unsigned step )
{
        std::array< uint8_t, N > steps;
        forEach( steps, [step]( uint8_t& arr_step ) { arr_step = uint8_t( step ); } );

        return generateLegGraph< LegTestGraph< N >, N >( steps );
}

template < unsigned N >
void removeIngoingEdges( LegTestGraph< N >& graph, const LegTestVertex< N >& node )
{
        forEach( graph.getVertexes(), [&]( LegTestVertex< N >& subnode ) {
                forEachIter( subnode.getEdges(), [&]( auto iter ) {
                        if ( iter->getTargetID() == node.getID() ) {
                                subnode.removeEdge( iter );
                        }
                } );
        } );
}

std::string drawSimplexes( const LegTestGraph< 2 >& graph, const SimplexStorage< LegVertexID, 2 >& storage )
{
        constexpr unsigned                                   scale = 2;
        std::map< unsigned, std::map< unsigned, unsigned > > area;

        forEach( storage, [&]( const Simplex< LegVertexID, 2 >& simplex ) {
                auto angles = mapF< std::vector< DiscreteAngles< 2 > > >( simplex, [&]( LegVertexID nid ) {
                        return graph.getVertex( nid )->angles;  //
                } );

                forEach( angles, [&]( const DiscreteAngles< 2 > a ) {
                        area[scale * a[0]][scale * a[1]] += 1;  //
                } );

                area[unsigned( angles[0][0] + angles[1][0] )][unsigned( angles[0][1] + angles[1][1] )] += 1;
                area[unsigned( angles[1][0] + angles[2][0] )][unsigned( angles[1][1] + angles[2][1] )] += 1;
                area[unsigned( angles[2][0] + angles[0][0] )][unsigned( angles[2][1] + angles[0][1] )] += 1;
        } );

        unsigned max_y = max( area, [&]( const std::pair< unsigned, std::map< unsigned, unsigned > >& pair ) {
                return pair.second.rbegin()->first;
        } );

        std::stringstream ss{};

        forEach( range(area.rbegin()->first + 1), [&]( std::size_t x ) {
                forEach( range(max_y + 1), [&]( std::size_t y ) {
                        unsigned val = area[unsigned( x )][unsigned( y )];

                        if ( y % 2 == 0 && x % 2 == 0 ) {
                                ss << "\033[4m";
                        } else {
                                ss << "\033[0m";
                        }
                        if ( val == 0 ) {
                                ss << "    ";
                        } else if ( val < 10 ) {
                                ss << " 0" << val << " ";
                        } else {
                                ss << " " << val << " ";
                        }
                } );
                ss << std::endl;
                ss << std::endl;
        } );
        return ss.str();
}
