#include "environment.h"
#include "robot.h"
#include "schpin_test/main.h"
#include <gtest/gtest.h>

namespace schpin
{

Either< std::tuple< Environment, Robot >, TestError > loadTestEnvRobot( std::filesystem::path env_file,
                                                                        std::filesystem::path robot_file )
{
        std::filesystem::path prefix{"./"};
        if ( ARGS.find( "--data-prefix" ) != ARGS.end() ) {
                prefix = ARGS["--data-prefix"];
        }
        std::ifstream env_data{prefix / env_file};
        std::ifstream robot_data{prefix / robot_file};
        return assembleLeftCollectRight(
                   safeLoadJson< Environment >( env_data ).convertRight( [&]( JsonStorageError e ) -> TestError {
                           return E( TestError ).attach( e ) << "Failed to load test env." << env_file;
                   } ),
                   safeLoadJson< Robot >( robot_data ).convertRight( [&]( JsonStorageError e ) -> TestError {
                           return E( TestError ).attach( e ) << "Failed to load test robot: " << robot_file;
                   } ) )
            .convertRight( []( std::vector< TestError > errors ) -> TestError {
                    return E( TestError ).attach( errors ) << "Failed to load test settings";
            } );
}

template < typename UnaryFunction >
void runIn( std::string env, UnaryFunction f )
{
        loadTestEnvRobot( env, "robot/koke.json" )
            .match( f,
                    [&]( TestError e ) {  //
                            FAIL() << e;
                    } );
}

}  // namespace schpin
