#include "base/line.h"
#include "schpin_test/point_test_util.h"
#include <gtest/gtest.h>

TYPED_TEST( PointTest, line_point_distance )
{
        static constexpr unsigned N = TypeParam::value;
        Line< N >                 line{this->zero_p_, this->unit_p_};

        ASSERT_NEAR( *linePointDistance( line, this->unit_p_ ), *Distance( 0. ), 1e-6 );
        ASSERT_NEAR( *linePointDistance( line, this->zero_p_ ), *Distance( 0. ), 1e-6 );
        ASSERT_NEAR( *linePointDistance( line, -this->unit_p_ ), *this->unit_p_.length(), 1e-6 );
}

TYPED_TEST( PointTest, perpendicular )
{
        static constexpr unsigned N = TypeParam::value;
        Point< 2 >                p{{N, 0}};
        Point< 2 >                zero_p{{0, 0}};
        Point< 2 >                unit_p{{N, N}};

        Distance diag_dist{float( sqrt( 2 * N * N ) / 2 )};

        Line< 2 > line{zero_p, unit_p};
        ASSERT_NEAR( *linePointDistance( line, p ), *diag_dist, 1e-6 );

        Line< 2 > reversed_line{unit_p, zero_p};
        ASSERT_NEAR( *linePointDistance( reversed_line, p ), *diag_dist, 1e-6 );

        Line< 2 > diag_line{Point< 2 >{{0, N}}, Point< 2 >{{N, 0}}};
        ASSERT_NEAR( *linePointDistance( diag_line, zero_p ), *diag_dist, 1e-6 );
        ASSERT_NEAR( *linePointDistance( diag_line, unit_p ), *diag_dist, 1e-6 );
}
