#include "base/point.h"
#include "schpin_test/point_test_util.h"
#include <gtest/gtest.h>

using namespace schpin;

TEST( Point, dot )
{
        Point< 3 > a{0, 1, 0};
        Point< 3 > b{-0.5, -0.5, 0};
        Point< 3 > c{1, 1, 0};

        ASSERT_EQ( a.dot( b ), -0.5 );
        ASSERT_EQ( c.dot( b ), -1 );
}

TYPED_TEST( PointTest, length )
{
        static constexpr unsigned N = TypeParam::value;
        ASSERT_EQ( *this->zero_p_.length(), 0.f );
        ASSERT_NEAR( *this->unit_p_.length(), sqrt( N ), 1e-6 );
        ASSERT_NEAR( *this->dim_p_.length(), sqrt( N * N * N ), 1e-6 );
}

TYPED_TEST( PointTest, normalized )
{
        ASSERT_NEAR( *this->dim_p_.normalized().length(), *this->unit_p_.normalized().length(), 1e-6 );
}

TYPED_TEST( PointTest, json )
{
        static constexpr unsigned N = TypeParam::value;

        nlohmann::json zero_p_j = nlohmann::json( this->zero_p_ );
        nlohmann::json unit_p_j = nlohmann::json( this->unit_p_ );
        nlohmann::json dim_p_j  = nlohmann::json( this->dim_p_ );

        ASSERT_EQ( zero_p_j.get< Point< N > >(), this->zero_p_ );
        ASSERT_EQ( unit_p_j.get< Point< N > >(), this->unit_p_ );
        ASSERT_EQ( dim_p_j.get< Point< N > >(), this->dim_p_ );
}

TEST( PointTestSimple, cross )
{
        Point< 3 > a{10, 255, 1};
        Point< 3 > b{-22, 0, 4};

        ASSERT_EQ( cross( a, b ).dot( a ), 0 );
        ASSERT_EQ( cross( a, b ).dot( b ), 0 );
}
