#include "base/pose.h"
#include <gtest/gtest.h>

using namespace schpin;

TEST( Pose, distance )
{
        Pose neutral;
        Pose pos_offseted{Point< 3 >( 10, 0, 0 )};
        Pose angle_offseted{Quaternion( Z_AXIS, Angle( 1. ) )};

        ASSERT_NEAR( neutral.distance( pos_offseted ).sumDistance(), 10, 1e-6 );
        ASSERT_NEAR( neutral.distance( angle_offseted ).sumDistance(), 1., 1e-6 );
}

TEST( Pose, interp )
{
        Pose neutral;
        Pose pos_offseted{Point< 3 >( 10, 0, 0 )};
        Pose angle_offseted{Quaternion( Z_AXIS, Angle( 1. ) )};

        Pose offseted{Point< 3 >( 10, 0, 0 ), Quaternion( Z_AXIS, Angle( 1. ) )};

        ASSERT_EQ( interp( neutral, pos_offseted, 0.5 ), Pose{Point< 3 >( 5, 0, 0 )} );
        ASSERT_TRUE( almostEqual( interp( neutral, angle_offseted, 0.5 ), Pose{Quaternion( Z_AXIS, Angle( 0.5 ) )} ) );

        Pose half_offseted{Point< 3 >( 5, 0, 0 ), Quaternion( Z_AXIS, Angle( 0.5 ) )};
        ASSERT_TRUE( almostEqual( interp( neutral, offseted, 0.5 ), half_offseted ) );
}
