
#include "base/triangle.h"
#include <gtest/gtest.h>

using namespace schpin;

void testTriangleSphereCenter( const Triangle< 3 >& triangle )
{

        Point< 3 > center = getTriangleSphereCenter( triangle );

        Distance scale = center.distance( triangle[0] );

        ASSERT_NEAR( *center.distance( triangle[0] ), *center.distance( triangle[1] ), double( *scale ) * 1e-6 );
        ASSERT_NEAR( *center.distance( triangle[0] ), *center.distance( triangle[2] ), double( *scale ) * 1e-6 );
}

TEST( TriangleSphereCenter, base )
{
        Triangle< 3 > triangle{Point< 3 >( 0, 0, 0 ), Point< 3 >( 1, 0, 0 ), Point< 3 >( 0, 1, 0 )};
        testTriangleSphereCenter( triangle );
}

TEST( TriangleSphereCenter, base2 )
{
        Triangle< 3 > triangle{Point< 3 >( -500, 200, -300 ), Point< 3 >( 6, 66, 666 ), Point< 3 >( 10, 10, 10 )};
        testTriangleSphereCenter( triangle );
}

TEST( Triangle, collisionQuery )
{
        Triangle< 3 >               t1{Point< 3 >( 0, 1, 0 ), Point< 3 >( 0, 0, 0 ), Point< 3 >( 1, 0, 0 )};
        TriangleCollisionQuery< 3 > q1( t1 );

        Point< 3 > normal = *q1.getSeparationAxes().begin();

        ASSERT_EQ( normal.dot( t1[0] ), 0 ) << normal;
        for ( const Point< 3 >& p : q1.getIntersectionAxes() ) {
                ASSERT_EQ( p.dot( normal ), 0 ) << p;
                ASSERT_TRUE( anyOf( t1, [&]( const Point< 3 >& t_point ) { return p.dot( t_point ) == 0; } ) );
        }
}
