#include "leg.h"
#include "leg/graph/gen.h"
#include "schpin_test/leg_test_util.h"
#include <gtest/gtest.h>

using namespace schpin;

TEST( Simplex, simple )
{
        LegTestGraph< 2 > graph = genLegTestGraph< 2 >( 2 );

        SimplexStorage< LegVertexID, 2 > storage = autogenerateSimplexes< LegTestGraph< 2 >, 2 >( graph );

        ASSERT_EQ( storage.size(), 2 ) << drawSimplexes( graph, storage );
        ASSERT_NE( storage[0], storage[1] );
}

TEST( Simplex, twoByTwo )
{
        LegTestGraph< 2 > graph = genLegTestGraph< 2 >( 3 );

        SimplexStorage< LegVertexID, 2 > storage = autogenerateSimplexes< LegTestGraph< 2 >, 2 >( graph );

        ASSERT_EQ( storage.size(), 8 ) << drawSimplexes( graph, storage );
}

TEST( Simplex, lShape )
{
        LegTestGraph< 2 > graph = genLegTestGraph< 2 >( 3 );

        auto node_iter = findIf( graph.getVertexes(), []( const LegTestVertex< 2 >& node ) -> bool {
                return node->angles[0] == 0 && node->angles[1] == 0;
        } );

        graph.purgeAllEdges( node_iter->getID() );

        SimplexStorage< LegVertexID, 2 > storage = autogenerateSimplexes< LegTestGraph< 2 >, 2 >( graph );

        ASSERT_EQ( storage.size(), 7 ) << drawSimplexes( graph, storage );
}

TEST( Simplex, oShape )
{
        LegTestGraph< 2 > graph = genLegTestGraph< 2 >( 5 );

        auto node_iter = findIf( graph.getVertexes(), []( const LegTestVertex< 2 >& node ) {
                return node->angles[0] == 2 && node->angles[1] == 2;
        } );

        graph.purgeAllEdges( node_iter->getID() );

        SimplexStorage< LegVertexID, 2 > storage = autogenerateSimplexes< LegTestGraph< 2 >, 2 >( graph );

        ASSERT_EQ( storage.size(), 28 ) << drawSimplexes( graph, storage );
}
