#include "schpin_test/main.h"
#include "ros/ros.h"
#include <gtest/gtest.h>
#include <schpin_lib/util.h>

namespace schpin
{
std::map< std::string, std::string > ARGS;
}

using namespace schpin;

Either< std::map< std::string, std::string >, TestError > parseArgs( int argc_in, char** argv_in )
{
        std::map< std::string, std::string > res;

        // skip the binary name
        argc_in -= 1;
        argv_in = std::next( argv_in );

        if ( argc_in % 2 != 0 ) {
                return {E( TestError ) << "Failed to parse test arguments, arg count: " << argc_in};
        }

        forEach( range( std::size_t( argc_in / 2 ) ), [&]( std::size_t i ) {
                std::string key = argv_in[2 * i];
                std::string val = argv_in[2 * i + 1];

                res[key] = val;
        } );

        if ( res.size() != std::size_t( argc_in / 2 ) ) {
                return {E( TestError ) << "Duplicate keys in arguments"};
        }

        bool keys_ok = allOf( res, [&]( const auto& map_pair ) { return map_pair.first.find( "--" ) == 0; } );
        if ( !keys_ok ) {
                return {E( TestError ) << "Failed to parse keys"};
        }

        return {res};
}

int main( int argc_in, char** argv_in )
{
        ROSCONSOLE_AUTOINIT;
        ros::Time::init();
        testing::InitGoogleTest( &argc_in, argv_in );

        auto arg_either = parseArgs( argc_in, argv_in );

        arg_either.match( [&]( std::map< std::string, std::string > args ) { ARGS = args; },
                          [&]( TestError e ) { std::cerr << e; } );

        if ( !arg_either.isLeft() ) {
                return -1;
        }

        return RUN_ALL_TESTS();
}
