#include "util.h"
#include <gtest/gtest.h>

using namespace schpin;

TEST( Util, assembleOptionals )
{
        auto either = assembleOptionals( std::optional< float >( 0.1f ), std::optional< float >() );
        ASSERT_FALSE( either.isLeft() );

        either = assembleOptionals( std::optional< float >( 0.1f ), std::optional< float >( 0.1f ) );
        ASSERT_TRUE( either.isLeft() );
}

TEST( Util, assembleLeftCollectRight )
{
        Either< std::tuple< int, unsigned >, std::vector< float > > either =
            assembleLeftCollectRight( Either< int, float >( float( 0.1 ) ), Either< unsigned, float >( float( 0.1 ) ) );

        ASSERT_FALSE( either.isLeft() );

        either =
            assembleLeftCollectRight( Either< int, float >( int( 1 ) ), Either< unsigned, float >( float( 0.1 ) ) );

        ASSERT_FALSE( either.isLeft() );

        either =
            assembleLeftCollectRight( Either< int, float >( int( 1 ) ), Either< unsigned, float >( unsigned( 1 ) ) );

        ASSERT_TRUE( either.isLeft() );
}

TEST( Util, partitionEithers )
{
        auto partitioned =
            partitionEithers< int, float >( {{int( 1 )}, {int( 2 )}, {int( 15 )}, {float( 0.1 )}, {float( -0.1 )}} );

        ASSERT_EQ( partitioned.left.size(), 3 );
        ASSERT_EQ( partitioned.right.size(), 2 );
}
