#include "environment.h"

using namespace schpin;

int main( int argc, char* argv[] )
{
        ros::Time::init();
        if ( argc != 4 ) {
                SCHPIN_WARN_LOG( "Bad number of arguments" );
        }

        std::string input_filename{argv[1]};
        double      edge_length = std::stod( argv[2] );
        std::string output_filename{argv[3]};

        EnvConfig config;

        config.default_occ     = ENV_MIN_OCCUPANCY;
        config.min_edge_length = Length( float( edge_length ) );
        config.paths.push_back( input_filename );

        buildEnvironment( config ).match(
            [&]( Environment&& env ) {
                    std::ofstream output{output_filename};
                    safeStoreJson( output, env ).optionally( []( JsonStorageError e ) { SCHPIN_WARN_LOG( e ); } );
            },
            [&]( EnvError e ) { SCHPIN_WARN_LOG( e ); } );

        return 0;
}
