#include "robot.h"

using namespace schpin;

int main( int argc, char* argv[] )
{

        ros::init( argc, argv, "planning_tester" );
        ros::NodeHandle handle{"~"};

        std::string output_filename;
        if ( !handle.getParam( "output_file", output_filename ) ) {
                SCHPIN_WARN_LOG( "No output file specified" );
                return -1;
        }

        urdf::Model model;
        model.initParam( "/robot_description" );

        std::string srdf;
        handle.getParam( "/robot_semantics", srdf );

        std::istringstream    srdf_istream{srdf};
        std::filesystem::path cache_prefix{"/home/squirrel/.schpin/cache/"};

        loadRobot( cache_prefix, model, srdf_istream )
            .match(
                [&]( Robot&& robot ) {
                        std::ofstream out_stream{output_filename};
                        SCHPIN_INFO_LOG( "Writing to " << output_filename );
                        if ( !out_stream ) {
                                SCHPIN_WARN_LOG( "Failed to open file: " << strerror( errno ) );
                                return;
                        }
                        safeStoreJson( out_stream, robot ).optionally( [&]( JsonStorageError e ) {
                                SCHPIN_WARN_LOG( e );
                        } );
                },
                []( RobotError e ) { SCHPIN_WARN_LOG( e ); } );

        ros::spinOnce();

        return 0;
}
