/**
 * Copyright 2017 by Jan Koniarik <jan.koniarik@gmail.com>
 * 
 * This file is part of Schpin.
 * 
 * Schpin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Schpin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Schpin.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @license LPGL-3.0+ <https://www.gnu.org/licenses/lgpl.html>
 */
#include "leg/config.h"
#include "leg/graph.h"
#include "leg/sidepath.h"

#pragma once

namespace schpin
{
class Leg
{
  leg::Graph graph_;
  std::map< leg::JointStates, Point< anchor::Leg > > states_to_point_;
  leg::Sidepath sidepath_;
  leg::Config   config_;

public:
  template < typename Graph >
  Leg( Graph&& graph, std::map< leg::JointStates, Point< anchor::Leg > > states_to_point, leg::Config config )
    : graph_( std::forward< leg::Graph >( graph ) )
    , states_to_point_( states_to_point )
    , sidepath_()
    , config_( config )
  {
  }

  Point< anchor::World > getSidepathPoint( const leg::SidepathDistance& dist ) const
  {
    return sidepath_.getPoint( dist );
  }

  Point< anchor::Leg > getPoint( const leg::JointStates& states ) const
  {
    return states_to_point_.at( states );
  }

  Point< anchor::Leg > getPoint( const leg::VertexID& vid ) const
  {
    const leg::JointStates& states = graph_.getVertex( vid ).getJointStates();
    return getPoint( states );
  }

  const leg::Graph& getGraph() const
  {
    return graph_;
  };

  const leg::Config& getConfig() const
  {
    return config_;
  };

  Pose getActualPose( const Pose& body_pose ) const
  {
    return body_pose;
  };
};
}
