/**
 * Copyright 2017 by Jan Koniarik <jan.koniarik@gmail.com>
 * 
 * This file is part of Schpin.
 * 
 * Schpin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Schpin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Schpin.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @license LPGL-3.0+ <https://www.gnu.org/licenses/lgpl.html>
 */
#include "../../util/identificator.h"
#include <cstdint>
#include <vector>

#pragma once

namespace schpin
{
namespace leg
{
  using util::Identificator;

  typedef double  Price;
  typedef uint8_t EdgeCount;

  typedef double                    JointAngle;
  typedef std::vector< JointAngle > JointAngles;

  typedef uint16_t                  JointState;  // TODO: make checks for overflow/underflow
  typedef std::vector< JointState > JointStates;

  std::ostream& operator<<( std::ostream& stream, const JointStates& joint_states )
  {
    stream << "Joint states: ";
    for ( std::size_t i = 0; i < joint_states.size(); ++i )
    {
      stream << i << ":" << joint_states[i] << " ";
    }
    return stream;
  }

  class VertexID : public Identificator< VertexID, 16 >
  {
    using Identificator< VertexID, 16 >::Identificator;
  };

  class EdgeID : public Identificator< EdgeID, 32 >
  {
    using Identificator< EdgeID, 32 >::Identificator;
  };
}
}
