/**
 * Copyright 2017 by Jan Koniarik <jan.koniarik@gmail.com>
 * 
 * This file is part of Schpin.
 * 
 * Schpin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Schpin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Schpin.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @license LPGL-3.0+ <https://www.gnu.org/licenses/lgpl.html>
 */
#include <boost/integer.hpp>
#include <ros/assert.h>

#pragma once

namespace schpin
{
namespace solver
{

  template < typename T, unsigned bits >
  class Axis
  {
  public:
    class Point
    {
    public:
      typedef typename boost::int_t< bits >::least ValueType;

      ValueType value_;

      // TODO: this should be named differently
      const ValueType& getValue() const
      {
        return value_;
      }
    };

    typedef double ValueType;

  private:
    typedef typename Point::ValueType PointValueType;

    PointValueType min_;
    PointValueType max_;

    double step_;

  public:
    Axis( intmax_t min, intmax_t max, double step )
      : min_( min )
      , max_( max )
      , step_( step )
    {
      ROS_ASSERT( min >= std::numeric_limits< PointValueType >::min() );
      ROS_ASSERT( max <= std::numeric_limits< PointValueType >::max() );
    }

    double getValue( const Point& point )
    {
      return point.getValue() * step_;
    }

    Point getMinPoint()
    {
      return Point( min_ );
    }

    Point getMaxPoint()
    {
      return Point( max_ );
    }

    Point getNextPoint( const Point& point )
    {
      ROS_ASSERT( point.getValue() < max_ );
      return Point( point.getValue() + 1 );
    }
    Point getPrevPoint( const Point& point )
    {
      ROS_ASSERT( point.getValue() > max_ );
      return Point( point.getValue() - 1 );
    }
  };

  class RollAxis : public Axis< RollAxis, 8 >
  {
  };
  class YawAxis : public Axis< YawAxis, 8 >
  {
  };
  class PitchAxis : public Axis< PitchAxis, 8 >
  {
  };
  class ZAxis : public Axis< ZAxis, 8 >
  {
  };
  class YAxis : public Axis< YAxis, 8 >
  {
  };
  class XAxis : public Axis< ZAxis, 16 >
  {
  };
}
}
