/**
 * Copyright 2017 by Jan Koniarik <jan.koniarik@gmail.com>
 * 
 * This file is part of Schpin.
 * 
 * Schpin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Schpin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Schpin.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @license LPGL-3.0+ <https://www.gnu.org/licenses/lgpl.html>
 */
#include "../basic.h"
#include "../leg/approx.h"
#include "robot.h"

namespace schpin
{
namespace solver
{

  // TODO: now, leg graph does not have 'load' axis that would scale the values based on load on leg
  //  - it's relevant if leg is shifting with body alone, or with other legs...

  Duration evaluateBodyShiftDuration( const State& start, const State& goal, const Robot& robot )
  {
    // evalutes time needed to shift body from 'start' state to 'goal' state

    Duration res{0};
    Pose     start_pose = projectStateToPose( start.getBodyState(), robot.getConfig() );
    Pose     goal_pose  = projectStateToPose( goal.getBodyState(), robot.getConfig() );

    std::for_each( start.getStaticLegs().begin(), start.getStaticLegs().end(), [&]( const StaticLeg& start_leg ) {

      auto iter = std::find_if( goal.getStaticLegs().begin(),
                                goal.getStaticLegs().end(),
                                [&]( const StaticLeg& goal_leg ) { return goal_leg.getID() == start_leg.getID(); } );

      if ( iter == goal.getStaticLegs().end() )
      {
        return;
      }

      const StaticLeg& goal_leg = *iter;
      ROS_ASSERT( goal_leg.getID() == start_leg.getID() );
      const Leg& leg = robot.getLeg( start_leg.getID() );

      Point< anchor::Leg > move_start = transformPoint( start_leg.getTipPosition(), start_pose, leg );
      Point< anchor::Leg > move_goal  = transformPoint( goal_leg.getTipPosition(), goal_pose, leg );

      const leg::Vertex vertex_hint = leg.getGraph().getVertex( start_leg.getVertexHintID() );

      Duration actual_duration = approxPointMovementDuration( leg, move_start, vertex_hint, move_goal );

      res = std::max( actual_duration, res );

    } );

    return res;
  }
}
}
