/**
 * Copyright 2017 by Jan Koniarik <jan.koniarik@gmail.com>
 * 
 * This file is part of Schpin.
 * 
 * Schpin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Schpin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Schpin.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @license LPGL-3.0+ <https://www.gnu.org/licenses/lgpl.html>
 */

#include "data/types.h"

#pragma once

namespace schpin
{
namespace data
{
  namespace graph
  {
    typedef BodyState VertexIndexKey;

    class IndexLeaf
    {
      typedef std::vector< VertexID >              VertexIDContainer;
      typedef typename VertexIDContainer::iterator IndexIterator;

      VertexIDContainer data_;

      void insert( const VertexIndexKey, VertexID vid )
      {
        data_.push_back( vid );
      }

      boost::iterator_range< IndexIterator > getVertexIDRange( const VertexIndexKey key )
      {
        return boost::iterator_range( data_.begin(), data_.end() );
      }
    };

    typedef typename IndexLeaf::IndexIterator IndexIterator;

    template < typename T, typedef IndexComponentT >
    class ArrayIndex
    {
      // TODO: can'thandle negative values of 'T"
      typedef IndexComponentT IndexComponent;

      std::array< T, IndexComponent::max - IndexComponent::min > data_;

      void insert( const VertexIndexKey key, VertexID vid )
      {
        data_[key.get< IndexComponent >()].insert( key, vid );
      }

      boost::iterator_range< IndexIterator > getVertexIDRange( const VertexIndexKey key )
      {
        return data_[key.get< IndexComponent >()].getVertexIDRange( key );
      }
    };

    template < typename T, typedef IndexComponentT >
    class VectorIndex
    {
      typedef IndexComponentT IndexComponent;

      std::vector< T > data_;

      void insert( const VertexIndexKey key, VertexID vid )
      {
        if ( key.get< IndexComponent >() >= data_.size() )
        {
          ROS_ASSERT( false );
        }

        data_[key.get< IndexComponent >()].insert( key, vid );
      }

      boost::iterator_range< IndexIterator > getVertexIDRange( const VertexIndexKey key )
      {
        ROS_ASSERT( key.get< IndexComponent >() < data_.size() );
        return data_[key.get< IndexComponent >()].getVertexIDRange( key );
      }
    };
  }
}
}
