/**
 * Copyright 2017 by Jan Koniarik <jan.koniarik@gmail.com>
 * 
 * This file is part of Schpin.
 * 
 * Schpin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Schpin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Schpin.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @license LPGL-3.0+ <https://www.gnu.org/licenses/lgpl.html>
 */
#include "src/util/identificator.h"

#pragma once

namespace schpin
{
namespace solver
{

  class VertexID : public Identificator< VertexID, 16 >
  {
    using Identificator< VertexID, 16 >::Identificator;
  };

  class EdgeIndex : public Identificator< EdgeID, 16 >
  {
    using Identificator< EdgeID, 16 >::Identificator;
  };

  typedef uint8_t EdgeCount;
  typedef double  Price;

  class VertexData
  {
    State state_;
  };

  class EdgeData
  {
    Price current_;
    Price time_;
    Price stability_;
  };

  class OutgoingEdge
  {
    EdgeData  data_;
    VertexID  target_;
    EdgeIndex ingoing_edge_i_;
  };

  class IngoingEdge
  {
    VertexID  source_;
    EdgeIndex outgoing_edge_i_;
  };

  class Edge
  {
    std::reference_wrapper< OutgoingEdge > oedge_;
    std::reference_wrapper< IngoingEdge >  iedge_;

    friend class Graph;

  public:
    const VertexID& getSourceID() const
    {
      return iedge_.get().getSourceID();
    }

    const VertexID& getTargetID() const
    {
      return oedge_.get().getTargetID();
    }

    EdgeData& refData()
    {
      return oedge_.get().refData();
    }
  }

  class VertexStorage
  {
    VertexData                  data_;
    std::vector< IngoingEdge >  ingoing_edges_;
    std::vector< OutgoingEdge > outgoing_edges_;

    std::vector< IngoingEdge >& refIngoingEdges()
    {
      return ingoing_edges_;
    }
  };

  class Vertex
  {
    VertexID      vid_;
    VertexStorage storage_;

  public:
    const VertexData& getData() const
    {
      return storage_.get().refData();
    }

    VertexData& refData()
    {
      return storage_.get().refData();
    }

    const VertexID& getID() const
    {
      return id_;
    }
  };
}
}
