/**
 * Copyright 2017 by Jan Koniarik <jan.koniarik@gmail.com>
 * 
 * This file is part of Schpin.
 * 
 * Schpin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Schpin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Schpin.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @license LPGL-3.0+ <https://www.gnu.org/licenses/lgpl.html>
 */
#include <memory>

#pragma once

namespace schpin
{
template < class T, class ValueType, class Identificator >
class LazyIterator
{
public:
  typedef ptrdiff_t                 difference_type;    // almost always ptrdiff_t
  typedef ValueType                 value_type;         // almost always T
  typedef value_type&               reference;          // almost always T& or const T&
  typedef value_type*               pointer;            // almost always T* or const T*
  typedef std::forward_iterator_tag iterator_category;  // usually std::forward_iterator_tag or similar

  Identificator                 actual_;
  std::shared_ptr< value_type > data_;

  LazyIterator( Identificator actual )
    : actual_( actual )
  {
  }

  // LazyIterator& operator=( const LazyIterator& );

  virtual value_type operator()( const Identificator& ) = 0;

  T& derived()
  {
    return *static_cast< T* >( this );
  }

  T& operator++()
  {
    ++actual_;
    data_ = nullptr;
    return derived();
  }

  T operator++(int)  // postfix increment
  {
    T copy( *this );
    this->operator++();
    return copy;
  }
  pointer operator->()
  {
    if ( !data_ )
    {
      data_ = std::make_shared< value_type >( this->operator()( actual_ ) );
    }

    return data_.get();
  }

  reference operator*()
  {
    return *this->operator->();
  }

  // TODO: I am not sure with this
  bool operator==( const T& rh ) const
  {
    return actual_ == rh.actual_;
  };
  bool operator!=( const T& rh ) const
  {
    return !( *this == rh );
  }

  virtual ~LazyIterator() = default;
};
}
