#include "./data.h"
#include "./util.h"
#include "graph/dijkstra.h"
#include "graph/util/iterator.h"
#include "leg/graph.h"
#include <gtest/gtest.h>
#include <utility>

using namespace schpin;

// TODO: in longterm, we want separate graph type for testing

typedef graph::DijkstraAlgorithm< leg::Graph > Algorithm;
typedef graph::Backtrack< leg::Vertex, leg::Edge > Backtrack;


TEST( DijsktraTest, simple )
{
  GraphData simple = getSimple();
  test::verifyVertexOrder< Algorithm >( simple );
}

TEST( DijsktraTest, star )
{

  GraphData star = getStar( 66 );
  test::verifyVertexOrder< Algorithm >( star );
}

TEST( DijsktraTest, line )
{

  GraphData line = getLine( 66 );

  auto iter = test::verifyVertexOrder< Algorithm >( line );

  --iter;

  Backtrack backtrack = iter.singleTrack();

  std::for_each( backtrack.begin(), backtrack.end(), [&]( leg::Vertex& vertex ) {
    ASSERT_EQ( vertex.getID(), line.vertex_order->front() );

    line.vertex_order->erase( line.vertex_order->begin() );
  } );

  ASSERT_TRUE( line.vertex_order->empty() );
}

TEST( DijsktraTest, petersen )
{
  GraphData petersen = getPetersen();

  test::verifyVertexOrder< Algorithm >( petersen );
};

// TODO: test with not reachable part of graph

int main( int argc, char** argv )
{
  testing::InitGoogleTest( &argc, argv );
  return RUN_ALL_TESTS();
}
