#include "graph/dstar.h"
#include "graph/dijkstra.h"
#include "leg/graph.h"
#include <boost/heap/d_ary_heap.hpp>
#include <gtest/gtest.h>
#include <limits>

using namespace schpin;

// TODO: make tests for graph and all priority queues

template < typename Item >
using HeapType = boost::heap::d_ary_heap< Item,
                                          boost::heap::arity< 2 >,
                                          boost::heap::mutable_< true >,
                                          boost::heap::compare< std::greater< Item > > >;

class HeuristicFunction
{
public:
  HeuristicFunction()
    : constant_( 0 )
  {
  }

  HeuristicFunction( leg::Price price )
    : constant_( price )
  {
  }

  leg::Price constant_;

  leg::Price operator()( leg::Vertex, leg::Vertex )
  {
    return constant_;
  }
};
typedef graph::DstarQueueItem< leg::Graph > QueueItem;
typedef graph::DstarAlgorithm< leg::Graph, HeapType< QueueItem >, HeuristicFunction > TestAlgorithm;

typedef typename TestAlgorithm::PriorityQueue TestPriorityQueue;

class DStarTest : public ::testing::Test
{
public:
  leg::Graph        g_{10};
  TestPriorityQueue t_queue_{g_};
  HeuristicFunction hf_;
};

TEST_F( DStarTest, simple )
{
  // prepare two-node, one edge graph
  auto v1 = g_.emplaceVertex();
  auto v2 = g_.emplaceVertex();

  g_.insertEdge( v1, v2, leg::EdgeData{} );

  TestAlgorithm s{g_, t_queue_, v1, v2, hf_};

  ASSERT_EQ( s.getSearchState(), graph::DstarSearchState::NOT_VALID );

  // make dstar search
  s.computeShortestPath();

  ASSERT_EQ( s.getSearchState(), graph::DstarSearchState::FINISHED );
  ASSERT_EQ( s.getBestSuccesor( v1 ).vertex, v2 );

  // add edge to make a loop
  g_.insertEdge( v1, v2, leg::EdgeData{} );

  // make dstar search again
  s.computeShortestPath();

  ASSERT_EQ( s.getBestSuccesor( v1 ).vertex, v2 );
}

int main( int argc, char** argv )
{
  testing::InitGoogleTest( &argc, argv );
  return RUN_ALL_TESTS();
}
