#include "leg/graph.h"
#include "basic.h"
#include <gtest/gtest.h>

using namespace schpin;

class GraphTest : public ::testing::Test
{
public:
  leg::Graph graph{leg::EdgeCount( 1 )};
};

TEST_F( GraphTest, simple )
{
  leg::Vertex v1 = graph.emplaceVertex();
  leg::Vertex v2 = graph.emplaceVertex();

  graph.insertEdge( v1, v2, leg::EdgeData{Duration( 0 )} );

  unsigned counter;

  counter = 0;
  std::for_each( graph.outgoingEdgeBegin( v1 ), graph.outgoingEdgeEnd( v1 ), [&]( leg::Edge edge ) {
    counter += 1;
    ASSERT_EQ( edge.getTargetID(), v2.getID() );
  } );
  ASSERT_EQ( counter, 1 );
  ASSERT_EQ( graph.ingoingEdgeBegin( v1 ), graph.ingoingEdgeEnd( v1 ) );

  ASSERT_EQ( graph.outgoingEdgeBegin( v2 ), graph.outgoingEdgeEnd( v2 ) );
  counter = 0;
  std::for_each( graph.ingoingEdgeBegin( v2 ), graph.ingoingEdgeEnd( v2 ), [&]( leg::Edge edge ) {
    counter += 1;
    ASSERT_EQ( edge.getSourceID(), v1.getID() );
  } );
  ASSERT_EQ( counter, 1 );
};

int main( int argc, char** argv )
{
  testing::InitGoogleTest( &argc, argv );
  return RUN_ALL_TESTS();
}
